@@ -1,6 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -40,11 +40,11 @@ void insert_lines( std::vector< std::string > & data,
 
 
 void delete_lines( std::vector< std::string > & data,
-                   const int line, const int lines ) throw()
+                   const int line, const int lines )
   {
   if( lines > 0 && line < (int)data.size() )
     {
-    for( unsigned int i = line + lines; i < data.size(); ++i )
+    for( unsigned i = line + lines; i < data.size(); ++i )
       data[i-lines].swap( data[i] );
     const int len = std::min( lines, (int)data.size() - line );
     data.erase( data.end() - len, data.end() );
@@ -67,7 +67,7 @@ void Basic_buffer::add_line( const char * const buf, const int len )
 
 
      // If all lines end with CR-LF, assume DOS file
-bool Basic_buffer::detect_and_remove_cr() throw()
+bool Basic_buffer::detect_and_remove_cr()
   {
   if( lines() <= 1 ) return false;
   for( int i = 0; i < last_line(); ++i )
@@ -82,7 +82,8 @@ bool Basic_buffer::detect_and_remove_cr() throw()
   }
 
 
-Basic_buffer::Basic_buffer( const Basic_buffer & b, const Point & p1, const Point & p2 )
+Basic_buffer::Basic_buffer( const Basic_buffer & b,
+                            const Point & p1, const Point & p2 )
   : data( 1 )
   {
   Point p = bof();
@@ -90,21 +91,21 @@ Basic_buffer::Basic_buffer( const Basic_buffer & b, const Point & p1, const Poin
   }
 
 
-bool Basic_buffer::blank( const int line ) const throw()
+bool Basic_buffer::blank( const int line ) const
   {
   Point p( line, 0 );
   return ( !pisvalid( p ) || bot( p ) == eol( p ) );
   }
 
 
-int Basic_buffer::characters( const int line ) const throw()
+int Basic_buffer::characters( const int line ) const
   {
-  if( line < 0 || line >= lines() ) return -1;
+  if( line < 0 || line >= lines() ) return 0;
   return data[line].size();
   }
 
 
-bool Basic_buffer::compare_lines( const int line1, const int line2 ) const throw()
+bool Basic_buffer::compare_lines( const int line1, const int line2 ) const
   {
   if( line1 < 0 || line1 >= lines() || line2 < 0 || line2 >= lines() )
     return false;
@@ -112,14 +113,14 @@ bool Basic_buffer::compare_lines( const int line1, const int line2 ) const throw
   }
 
 
-bool Basic_buffer::one_character() const throw()
+bool Basic_buffer::one_character() const
   {
   return ( data[0].size() == 1 &&
-           ( lines() == 1 || ( lines() == 2 && data[1].size() == 0 ) ) );
+           ( lines() == 1 || ( lines() == 2 && data[1].empty() ) ) );
   }
 
 
-int Basic_buffer::operator[]( const Point & p ) const throw()
+int Basic_buffer::operator[]( const Point & p ) const
   {
   if( p.col < 0 || p.col >= characters( p.line ) ) return -1;
   unsigned char ch = data[p.line][p.col];
@@ -127,7 +128,7 @@ int Basic_buffer::operator[]( const Point & p ) const throw()
   }
 
 
-int Basic_buffer::pgetc( Point & p ) const throw()
+int Basic_buffer::pgetc( Point & p ) const
   {
   Point p1 = p;
   if( !pisvalid( p ) || !pnext( p ) ) return -1;
@@ -136,7 +137,7 @@ int Basic_buffer::pgetc( Point & p ) const throw()
   }
 
 
-int Basic_buffer::pngetc( Point & p ) const throw()
+int Basic_buffer::pngetc( Point & p ) const
   {
   if( !pnext( p ) || p == eof() ) return -1;
   unsigned char ch = data[p.line][p.col];
@@ -144,7 +145,7 @@ int Basic_buffer::pngetc( Point & p ) const throw()
   }
 
 
-int Basic_buffer::ppgetc( Point & p ) const throw()
+int Basic_buffer::ppgetc( Point & p ) const
   {
   if( !pprev( p ) ) return -1;
   unsigned char ch = data[p.line][p.col];
@@ -152,7 +153,7 @@ int Basic_buffer::ppgetc( Point & p ) const throw()
   }
 
 
-bool Basic_buffer::pnext( Point & p ) const throw()
+bool Basic_buffer::pnext( Point & p ) const
   {
   if( ++p.col < characters( p.line ) || p == eof() ) return true;
   if( ++p.line < lines() ) { p.col = 0; return true; }
@@ -160,18 +161,18 @@ bool Basic_buffer::pnext( Point & p ) const throw()
   }
 
 
-bool Basic_buffer::pprev( Point & p ) const throw()
+bool Basic_buffer::pprev( Point & p ) const
   {
   int c = characters( p.line );
-  if( c >= 0 && p.col > c ) p.col = c;
+  if( p.col > c ) p.col = c;
   if( --p.col >= 0 ) return true;
-  int col = characters( --p.line ) - 1;
-  if( col >= 0 ) { p.col = col; return true; }
+  c = characters( --p.line ) - 1;
+  if( p.line >= 0 && c >= 0 ) { p.col = c; return true; }
   ++p.line; ++p.col; return false;
   }
 
 
-bool Basic_buffer::pseek( Point & p, const int n ) const throw()
+bool Basic_buffer::pseek( Point & p, const int n ) const
   {
   Point saved_p = p;
   p.col += n;
@@ -179,7 +180,7 @@ bool Basic_buffer::pseek( Point & p, const int n ) const throw()
     {
     while( p.line < last_line() )
       {
-      int c = characters( p.line );
+      const int c = characters( p.line );
       if( p.col < c || p == eof() ) break;
       p.col -= c; ++p.line;
       }
@@ -190,12 +191,12 @@ bool Basic_buffer::pseek( Point & p, const int n ) const throw()
     while( p.line > 0 && p.col < 0 ) p.col += characters( --p.line );
     if( p.line >= 0 && p.col >= 0 ) return true;
     }
-  else return true;	// n == 0
+  else return true;				// n == 0
   p = saved_p; return false;
   }
 
 
-bool Basic_buffer::pvalid( Point & p ) const throw()
+bool Basic_buffer::pvalid( Point & p ) const
   {
   if( p < bof() ) p = bof();
   else if( p > eof() ) p = eof();
@@ -255,7 +256,7 @@ bool Basic_buffer::pputc( Point & p, const unsigned char ch )
 
 
 // change char and move next
-bool Basic_buffer::pchgc( Point & p, const unsigned char ch ) throw()
+bool Basic_buffer::pchgc( Point & p, const unsigned char ch )
   {
   int ch1 = (*this)[p];
   if( ch == '\n' || ch1 < 0 || ch1 == '\n' ) return false;
@@ -285,7 +286,7 @@ bool Basic_buffer::pdelb( const Point & p1, const Point & p2 )
   }
 
 
-bool Basic_buffer::capitalize( const Point & p1, const Point & p2 ) throw()
+bool Basic_buffer::capitalize( const Point & p1, const Point & p2 )
   {
   bool change = false;
   bool prev_is_alnum = false;
@@ -305,7 +306,7 @@ bool Basic_buffer::capitalize( const Point & p1, const Point & p2 ) throw()
   }
 
 
-bool Basic_buffer::to_lowercase( const Point & p1, const Point & p2 ) throw()
+bool Basic_buffer::to_lowercase( const Point & p1, const Point & p2 )
   {
   bool change = false;
   if( p1 < p2 && pisvalid( p1 ) && pisvalid( p2 ) )
@@ -319,7 +320,7 @@ bool Basic_buffer::to_lowercase( const Point & p1, const Point & p2 ) throw()
   }
 
 
-bool Basic_buffer::to_uppercase( const Point & p1, const Point & p2 ) throw()
+bool Basic_buffer::to_uppercase( const Point & p1, const Point & p2 )
   {
   bool change = false;
   if( p1 < p2 && pisvalid( p1 ) && pisvalid( p2 ) )
@@ -334,7 +335,7 @@ bool Basic_buffer::to_uppercase( const Point & p1, const Point & p2 ) throw()
 
 
 // Is p at beginning of word?
-bool Basic_buffer::pisbow( const Point & p ) const throw()
+bool Basic_buffer::pisbow( const Point & p ) const
   {
   Point p1 = p;
   int ch = (*this)[p];
@@ -344,7 +345,7 @@ bool Basic_buffer::pisbow( const Point & p ) const throw()
 
 
 // Is p at end of word?
-bool Basic_buffer::piseow( const Point & p ) const throw()
+bool Basic_buffer::piseow( const Point & p ) const
   {
   Point p1 = p;
   int ch = ppgetc( p1 );
@@ -353,7 +354,7 @@ bool Basic_buffer::piseow( const Point & p ) const throw()
   }
 
 
-Point Basic_buffer::bol( const Point & p ) const throw()
+Point Basic_buffer::bol( const Point & p ) const
   {
   if( p.line >= lines() ) return eof();
   else if( p.line < 0 ) return bof();
@@ -361,7 +362,7 @@ Point Basic_buffer::bol( const Point & p ) const throw()
   }
 
 
-Point Basic_buffer::eol( const Point & p ) const throw()
+Point Basic_buffer::eol( const Point & p ) const
   {
   if( p.line >= last_line() ) return eof();
   else if( p.line < 0 ) return eol( bof() );
@@ -370,28 +371,29 @@ Point Basic_buffer::eol( const Point & p ) const throw()
 
 
 // returns position of first non whitespace character in line (eol if none)
-Point Basic_buffer::bot( const Point & p ) const throw()
+Point Basic_buffer::bot( const Point & p ) const
   {
   Point p1 = bol( p );
   const std::string & s = data[p1.line];
-  while( p1.col < (int)s.size() && std::isspace( s[p1.col] ) && s[p1.col] != '\n' )
+  while( p1.col < (int)s.size() &&
+         ISO_8859::isspace( s[p1.col] ) && s[p1.col] != '\n' )
     ++p1.col;
   return p1;
   }
 
 
 // returns position following last non whitespace character in line (bol if none)
-Point Basic_buffer::eot( const Point & p ) const throw()
+Point Basic_buffer::eot( const Point & p ) const
   {
   Point p1 = eol( p );
   const std::string & s = data[p1.line];
-  while( p1.col > 0 && std::isspace( s[p1.col-1] ) ) --p1.col;
+  while( p1.col > 0 && ISO_8859::isspace( s[p1.col-1] ) ) --p1.col;
   return p1;
   }
 
 
 // begin of paragraph
-Point Basic_buffer::bop( const Point & p ) const throw()
+Point Basic_buffer::bop( const Point & p ) const
   {
   Point p1 = p;
   if( !blank( p1.line ) ) while( !blank( p1.line - 1 ) ) --p1.line;
@@ -400,7 +402,7 @@ Point Basic_buffer::bop( const Point & p ) const throw()
 
 
 // end of paragraph
-Point Basic_buffer::eop( const Point & p ) const throw()
+Point Basic_buffer::eop( const Point & p ) const
   {
   Point p1 = p;
   while( !blank( p1.line ) ) ++p1.line;
@@ -414,7 +416,7 @@ Point Basic_buffer::eop( const Point & p ) const throw()
 // Returns +1 if done, 0 if [p] is not a delimiter, -1 if no matching delimiter.
 //
 int Basic_buffer::set_to_matching_delimiter( Point & p, bool forward,
-                                             const bool force ) const throw()
+                                             const bool force ) const
   {
   int ch1 = (*this)[p];
   if( ch1 < 0 ) return 0;
@@ -447,11 +449,10 @@ int Basic_buffer::set_to_matching_delimiter( Point & p, bool forward,
     if( ch >= 0 && ch == ch2 )
       {
       if( forward )
-        { if( pngetc( p1 ) >= 0 && find_text( p1, std::string( "*/" ) ) )
+        { if( pngetc( p1 ) >= 0 && find_text( p1, "*/" ) )
           { pseek( p1, -2 ); p = p1; return +1; } }
       else
-        { p1 = p; if( rfind_text( p1, std::string( "/*" ) ) )
-          { p = p1; return +1; } }
+        { p1 = p; if( rfind_text( p1, "/*" ) ) { p = p1; return +1; } }
       return -1;
       }
     return 0;
@@ -520,9 +521,9 @@ int Basic_buffer::set_to_matching_delimiter( Point & p, bool forward,
 // Search forward from 'p' for 'pattern' (Boyer-Moore algorithm)
 // If found, set 'p' after found substring
 //
-bool Basic_buffer::find_text( Point & p, const std::string & text, const bool icase ) const throw()
+bool Basic_buffer::find_text( Point & p, const std::string & text, const bool icase ) const
   {
-  if( text.size() == 0 ) return true;
+  if( text.empty() ) return true;
   Point p1 = p;
   if( !pseek( p1, text.size() ) ) return false;
   const std::string * textp = &text;
@@ -530,13 +531,13 @@ bool Basic_buffer::find_text( Point & p, const std::string & text, const bool ic
 
   if( icase )
     {
-    for( unsigned int i = 0; i < text.size(); ++i )
+    for( unsigned i = 0; i < text.size(); ++i )
       itext += ISO_8859::tolower( text[i] );
     textp = &itext;
     }
 
   std::vector< int > table( 256, textp->size() );
-  for( unsigned int i = 0; i < textp->size() - 1; ++i )
+  for( unsigned i = 0; i < textp->size() - 1; ++i )
     { unsigned char ch = (*textp)[i]; table[ch] = textp->size() - i - 1; }
 
   while( true )
@@ -560,9 +561,9 @@ bool Basic_buffer::find_text( Point & p, const std::string & text, const bool ic
 // Search backward from 'p-1' for 'pattern' (Boyer-Moore algorithm)
 // If found, set 'p' to the first char of found substring
 //
-bool Basic_buffer::rfind_text( Point & p, const std::string & text, const bool icase ) const throw()
+bool Basic_buffer::rfind_text( Point & p, const std::string & text, const bool icase ) const
   {
-  if( text.size() == 0 ) return true;
+  if( text.empty() ) return true;
   Point p1 = p;
   if( !pseek( p1, -text.size() ) ) return false;
   const std::string * textp = &text;
@@ -570,19 +571,19 @@ bool Basic_buffer::rfind_text( Point & p, const std::string & text, const bool i
 
   if( icase )
     {
-    for( unsigned int i = 0; i < text.size(); ++i )
+    for( unsigned i = 0; i < text.size(); ++i )
       itext += ISO_8859::tolower( text[i] );
     textp = &itext;
     }
 
   std::vector< int > table( 256, textp->size() );
-  for( unsigned int i = textp->size() - 1; i > 0; --i )
+  for( unsigned i = textp->size() - 1; i > 0; --i )
     { unsigned char ch = (*textp)[i]; table[ch] = i; }
 
   while( true )
     {
     Point p2 = p1;
-    unsigned int i; int delta = 0;
+    unsigned i; int delta = 0;
     for( i = 0; i < textp->size(); ++i )
       {
       unsigned char ch = (*textp)[i];
@@ -597,38 +598,45 @@ bool Basic_buffer::rfind_text( Point & p, const std::string & text, const bool i
   }
 
 
-Point Basic_buffer::to_cursor( const Point & pointer ) const throw()
+Point Basic_buffer::to_cursor( const Point & pointer ) const
   {
   Point p = bol( pointer );
   Point cursor = p;
-  while( p.col < pointer.col )
+  if( p.line >= 0 && p.line < lines() )
     {
-    int ch = (*this)[p];
-    if( ch < 0 ) break;
-    if( ch == '\t' ) cursor.col += 8 - ( cursor.col % 8 );
-    else ++cursor.col;
-    ++p.col;
+    const std::string & s = data[p.line];
+    const int limit = std::min( pointer.col, (int)s.size() );
+    while( p.col < limit )
+      {
+      int i = p.col;
+      while( i < limit && s[i] != '\t' ) ++i;
+      cursor.col += i - p.col;
+      if( i < limit ) cursor.col += 8 - ( cursor.col % 8 );
+      p.col = i + 1;
+      }
     }
   return cursor;
   }
 
 
-Point Basic_buffer::to_pointer( const Point & cursor, Point & pcursor ) const throw()
+Point Basic_buffer::to_pointer( const Point & cursor, Point & pcursor ) const
   {
   pcursor = bol( cursor );
   Point pointer = pcursor;
-  const int peolcol = eol( pointer ).col;
-  while( pcursor.col < cursor.col && pointer.col < peolcol )
+  if( pointer.line >= 0 && pointer.line < lines() )
     {
-    int ch = (*this)[pointer];
-    if( ch < 0 ) break;
-    if( ch == '\t' )
+    const std::string & s = data[pointer.line];
+    const int peolcol = eol( pointer ).col;
+    while( pcursor.col < cursor.col && pointer.col < peolcol )
       {
-      int col = pcursor.col + 8 - ( pcursor.col % 8 );
-      if( col <= cursor.col ) pcursor.col = col; else break;
+      if( s[pointer.col] == '\t' )
+        {
+        int col = pcursor.col + 8 - ( pcursor.col % 8 );
+        if( col <= cursor.col ) pcursor.col = col; else break;
+        }
+      else ++pcursor.col;
+      ++pointer.col;
       }
-    else ++pcursor.col;
-    ++pointer.col;
     }
   return pointer;
   }
@@ -652,3 +660,25 @@ int Basic_buffer::to_string( const Point & p1, const Point & p2,
     }
   return text.size();
   }
+
+
+int Basic_buffer::to_string( const Point & p1, const int size,
+                             std::string & text ) const
+  {
+  text.clear();
+  if( pisvalid( p1 ) && size > 0 )
+    {
+    int rest = size;
+    int line = p1.line;
+    const int sz = std::min( rest, (int)data[line].size() - p1.col );
+    text.assign( data[line], p1.col, sz );
+    rest -= sz;
+    while( rest > 0 && ++line < lines() )
+      {
+      const int sz = std::min( rest, (int)data[line].size() );
+      text.append( data[line], 0, sz );
+      rest -= sz;
+      }
+    }
+  return text.size();
+  }
