@@ -1,6 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
-    Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -59,6 +59,9 @@ std::vector< Point * > & get_marks_and_pointers( Buffer & b )
 } // end namespace
 
 
+const std::string Buffer::unnamed( "(Unnamed)" );
+
+
 bool Buffer::Change_history::Record::append_record( const Basic_buffer & buffer,
                                                     Record & r )
   {
@@ -155,9 +158,9 @@ Buffer::Buffer( const Options & opts, const std::string * const namep )
   : backuped( false ), modified_( false ), saved( false ),
     options( opts ), crlf( false )
   {
-  FILE *f = 0;
+  FILE * f = 0;
   errno = 0;
-  if( ( !namep || !namep->size() || *namep == "-" ) && !isatty( fileno( stdin ) ) )
+  if( ( !namep || namep->empty() || *namep == "-" ) && !isatty( fileno( stdin ) ) )
     f = stdin;
   else if( namep && namep->size() )
     { name_ = *namep; f = std::fopen( name_.c_str(), "r" ); }
@@ -196,15 +199,27 @@ Buffer::Buffer( const Options & opts, const std::string * const namep )
   if( file_error ) throw Error( "Error reading file" );
   if( !file_eof ) throw Error( "Not enough memory" );
   crlf = detect_and_remove_cr();
-  for( int i = 10; i < 60 && i < lines(); ++i )
-    if( !get_line( i ).compare( 0, 10, "#include \"" ) )
-      { mark[0].line = i - 1; break; }
+  set_marks();
+  }
+
+
+int Buffer::set_marks()
+  {
+  int c = 0;
   if( mark[0].line < 0 )
-    for( int i = 19; i < 30 && i < lines(); ++i )
-      if( blank( i ) ) { mark[0].line = i; break; }
-  if( mark[0].line < 0 ) mark[0].line = 22;
-  for( int i = 1; i < 10 && lines() >= 100 * i; ++i )
-    mark[i].line = 100 * i - 1;
+    {
+    for( int i = 10; i < 60 && i < lines(); ++i )
+      if( get_line( i ).compare( 0, 10, "#include \"" ) == 0 )
+        { mark[0].line = i - 1; ++c; break; }
+    if( mark[0].line < 0 )
+      for( int i = 19; i < 30 && i < lines(); ++i )
+        if( blank( i ) ) { mark[0].line = i; ++c; break; }
+    if( mark[0].line < 0 && lines() > 22 ) { mark[0].line = 22; ++c; }
+    }
+  for( int i = 1; i < 10 && 100 * i <= lines(); ++i )
+    if( mark[i].line < 0 )
+      { mark[i].line = 100 * i - 1; ++c; }
+  return c;
   }
 
 
@@ -226,7 +241,7 @@ bool Buffer::save( const std::string * namep, bool set_name,
       {
       if( name_ != *namep )
         {
-        FILE *f = std::fopen( namep->c_str(), "r+" );
+        FILE * f = std::fopen( namep->c_str(), "r+" );
         if( f )
           {
           std::fclose( f );
@@ -234,10 +249,10 @@ bool Buffer::save( const std::string * namep, bool set_name,
           if( key != 'y' && key != 'Y' ) return false;
           }
         }
-      if( !name_.size() ) name_ = *namep;
+      if( name_.empty() ) name_ = *namep;
       else if( name_ != *namep ) same_name = false;
       }
-    else { if( !name_.size() ) return false; else namep = &name_; }
+    else { if( name_.empty() ) return false; else namep = &name_; }
     }
   if( !namep || ( same_name && options.read_only ) ) return false;
 
@@ -255,7 +270,7 @@ bool Buffer::save( const std::string * namep, bool set_name,
     else std::remove( namep->c_str() );
     }
 
-  FILE *f = std::fopen( namep->c_str(), append ? "a" : "w" );
+  FILE * f = std::fopen( namep->c_str(), append ? "a" : "w" );
   if( !f )
     { if( !had_name ) { name_.clear(); } throw Error( "Error opening file" ); }
   if( mode ) fchmod( fileno( f ), mode );
@@ -284,7 +299,7 @@ bool Buffer::save( const std::string * namep, bool set_name,
   }
 
 
-long Buffer::calculate_offset( const Point & p, int & percent ) const
+long Buffer::character_offset( const Point & p, int & percent ) const
   {
   long total = 0;
 
@@ -300,73 +315,6 @@ long Buffer::calculate_offset( const Point & p, int & percent ) const
   }
 
 
-void Buffer::character_info( const Point & p, std::string & s ) const
-  {
-  char buf1[64], buf2[64];
-  const int ch = (*this)[p];
-
-  if( ch >= 0 )
-    snprintf( buf2, sizeof buf2, "Code %d(0x%02X/0%o)", ch, ch ,ch );
-  else if( p == eof() )
-    snprintf( buf2, sizeof buf2, "(End Of File)" );
-  else
-    { snprintf( buf2, sizeof buf2, "*** Error: Bad Pointer: Line %d  Pos %d",
-                p.line + 1, p.col + 1 ); s = buf2; return; }
-
-  int percent = 0;
-  long offset = calculate_offset( p, percent );
-
-  snprintf( buf1, sizeof buf1, "Line %d  Pos %d  Offset %ld(0x%lX)  %d%%  ",
-            p.line + 1, p.col + 1, offset, offset, percent );
-  s = buf1; s += buf2;
-  }
-
-
-void Buffer::multichar_value( const Point & p, std::string & s,
-                              const int size, const bool big_endian ) const
-  {
-  int bufsize = 0;
-  char * const buf = Screen::line_buf( &bufsize );
-  if( size < 2 || size > 8 )
-    {
-    snprintf( buf, bufsize, "*** Error: Bad Multichar Size: %d", size );
-    s = buf; return;
-    }
-  std::string text;
-  const int sz = to_string( p, size, text );
-  int percent = 0;
-  long offset = calculate_offset( p, percent );
-
-  if( sz >= size )
-    {
-    long long value = 0;
-    if( big_endian )
-      for( int i = 0; i < size; ++i )
-        { value <<= 8; value += (unsigned char)text[i]; }
-    else
-      for( int i = size - 1; i >= 0; --i )
-        { value <<= 8; value += (unsigned char)text[i]; }
-    snprintf( buf, bufsize, "Off %ld(0x%lX)  %s_en_%d %lld(0x%0*llX)",
-              offset, offset, big_endian ? "Big" : "Lit",
-              size, value, 2 * size, value );
-    }
-  else if( sz > 0 )
-    snprintf( buf, bufsize, "Offset %ld(0x%lX)  %d%%  (Only %d Characters Left)",
-              offset, offset, percent, sz );
-  else
-    snprintf( buf, bufsize, "Offset %ld(0x%lX)  %d%%  (End Of File)",
-              offset, offset, percent );
-  s = buf;
-  }
-
-
-const std::string & Buffer::nename() const
-  {
-  static const std::string unnamed( "(Unnamed)" );
-  if( name_.size() ) return name_; else return unnamed;
-  }
-
-
 // insert block b.[p1,p2) at p and move next
 bool Buffer::pputb( Point & p, const Basic_buffer & b, const Point & p1,
                     const Point & p2, const int save_change )
@@ -377,7 +325,7 @@ bool Buffer::pputb( Point & p, const Basic_buffer & b, const Point & p1,
   Basic_buffer::pputb( p, b, p1, p2 );
   if( save_change )		// save_change == 0 only for undo/redo
     {
-    const Point & after = (save_change != 2) ? p0 : p;
+    const Point & after = ( save_change != 2 ) ? p0 : p;
     Change_history::Record r( p0, after, modified_ );
     r.add_atom( Change_history::Atom( p0, p ) );
     change_history.add_record( *this, r );
@@ -481,7 +429,7 @@ bool Buffer::pdelb( const Point & p1, const Point & p2, const int save_change )
     return false;
   if( save_change )		// save_change == 0 only for undo/redo
     {
-    const Point & p0 = (save_change != 2) ? p1 : p2;
+    const Point & p0 = ( save_change != 2 ) ? p1 : p2;
     Change_history::Record r( p0, p1, modified_ );
     r.add_atom( Change_history::Atom( p1, p2, *this ) );
     change_history.add_record( *this, r );
