@@ -1,5 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,31 +26,31 @@ struct Point
   {
   int line, col;
 
-  Point( int l = -1, int c = 0 ) throw() : line( l ), col( c ) {}
+  explicit Point( int l = -1, int c = 0 ) : line( l ), col( c ) {}
 
-  Point & operator+=( const Point & p ) throw()
+  Point & operator+=( const Point & p )
     { line += p.line; col += p.col; return *this; }
-  Point & operator-=( const Point & p ) throw()
+  Point & operator-=( const Point & p )
     { line -= p.line; col -= p.col; return *this; }
 
-  Point operator+( const Point & p ) const throw()
+  Point operator+( const Point & p ) const
     { Point r( *this ); r += p; return r; }
-  Point operator-( const Point & p ) const throw()
+  Point operator-( const Point & p ) const
     { Point r( *this ); r -= p; return r; }
 
-  bool operator==( const Point & p ) const throw()
+  bool operator==( const Point & p ) const
     { return ( line == p.line && col == p.col ); }
-  bool operator!=( const Point & p ) const throw() { return !( *this == p ); }
+  bool operator!=( const Point & p ) const { return !( *this == p ); }
 
-  bool operator<( const Point & p ) const throw()
+  bool operator<( const Point & p ) const
     { return ( line < p.line || ( line == p.line && col < p.col ) ); }
-  bool operator>( const Point & p ) const throw()
+  bool operator>( const Point & p ) const
     { return ( line > p.line || ( line == p.line && col > p.col ) ); }
 
-  bool operator<=( const Point & p ) const throw() { return !( *this > p ); }
-  bool operator>=( const Point & p ) const throw() { return !( *this < p ); }
+  bool operator<=( const Point & p ) const { return !( *this > p ); }
+  bool operator>=( const Point & p ) const { return !( *this < p ); }
 
-  Point & cut() throw()
+  Point & cut()
     { if( line < 0 ) line = 0; if( col < 0 ) col = 0; return *this; }
   };
 
@@ -59,74 +60,80 @@ struct Point
 //
 class Basic_buffer
   {
-  std::vector< std::string > data;	// the lines
+  std::vector< std::string > data;		// the lines
+
+  Basic_buffer( const Basic_buffer & );		// declared as private
+  void operator=( const Basic_buffer & );	// declared as private
 
 protected:
   void add_line( const char * const buf, const int len );
-  bool detect_and_remove_cr() throw();
+  bool detect_and_remove_cr();
 
 public:
   Basic_buffer() : data( 1 ) {}
   Basic_buffer( const Basic_buffer & b, const Point & p1, const Point & p2 );
 
-  bool blank( const int line ) const throw()
-    { Point p( line, 0 ); return ( !pisvalid( p ) || bot( p ) == eol( p ) ); }
-  int  characters( const int line ) const throw();
-  bool compare_lines( const int line1, const int line2 ) const throw();
-  bool empty() const throw() { return ( data.size() == 1 && !data[0].size() ); }
-  int  lines() const throw() { return data.size(); }
-  int  last_line() const throw() { return lines() - 1; }
-  bool one_character() const throw();		// buffer has only one character
+  bool blank( const int line ) const;	// true if line is blank
+  int  characters( const int line ) const;
+  bool compare_lines( const int line1, const int line2 ) const;
+  bool empty() const { return ( data.size() == 1 && data[0].empty() ); }
+  int  lines() const { return data.size(); }
+  int  last_line() const { return lines() - 1; }
+  bool one_character() const;		// buffer has only one character
 
-  int  operator[]( const Point & p ) const throw();
-  int  pgetc( Point & p ) const throw();	// get and move next
-  int  pngetc( Point & p ) const throw();	// move next and get
-  int  ppgetc( Point & p ) const throw();	// move prev and get
-  bool pnext( Point & p ) const throw();
-  bool pprev( Point & p ) const throw();
-  bool pseek( Point & p, int n ) const throw();
-  bool pvalid( Point & p ) const throw();
+  const std::string & get_line( const int line ) const { return data[line]; }
+  int  operator[]( const Point & p ) const;
+  int  pgetc( Point & p ) const;	// get and move next
+  int  pngetc( Point & p ) const;	// move next and get
+  int  ppgetc( Point & p ) const;	// move prev and get
+  bool pnext( Point & p ) const;
+  bool pprev( Point & p ) const;
+  bool pseek( Point & p, const int n ) const;
+  bool pvalid( Point & p ) const;
 
   // insert block b.[p1,p2) at p and move next
   bool pputb( Point & p, const Basic_buffer & b, const Point & p1, const Point & p2 );
   // insert char and move next
   bool pputc( Point & p, const unsigned char ch );
   // change char and move next
-  bool pchgc( Point & p, const unsigned char ch ) throw();
+  bool pchgc( Point & p, const unsigned char ch );
   // delete block at [p1,p2)
   bool pdelb( const Point & p1, const Point & p2 );
-  bool to_lowercase( const Point & p1, const Point & p2 ) throw();
-  bool to_uppercase( const Point & p1, const Point & p2 ) throw();
+  bool capitalize( const Point & p1, const Point & p2 );
+  bool to_lowercase( const Point & p1, const Point & p2 );
+  bool to_uppercase( const Point & p1, const Point & p2 );
 
-  bool pisbow( const Point & p ) const throw();
-  bool piseow( const Point & p ) const throw();
-  bool pisvalid( const Point & p ) const throw()
+  bool pisbow( const Point & p ) const;
+  bool piseow( const Point & p ) const;
+  bool pisvalid( const Point & p ) const
     { return ( ( p.col >= 0 && p.col < characters( p.line ) ) || p == eof() ); }
 
-  Point bof() const throw() { return Point( 0, 0 ); }
-  Point eof() const throw() { return Point( last_line(), data[last_line()].size() ); }
-  Point bol( const Point & p ) const throw();
-  Point eol( const Point & p ) const throw();
-  Point bot( const Point & p ) const throw();	// begin of text in line
-  Point eot( const Point & p ) const throw();	// end of text in line
-  Point bop( const Point & p ) const throw();	// begin of paragraph
-  Point eop( const Point & p ) const throw();	// end of paragraph
+  Point bof() const { return Point( 0, 0 ); }
+  Point eof() const { return Point( last_line(), data[last_line()].size() ); }
+  Point bol( const Point & p ) const;
+  Point eol( const Point & p ) const;
+  Point bot( const Point & p ) const;	// begin of text in line
+  Point eot( const Point & p ) const;	// end of text in line
+  Point bop( const Point & p ) const;	// begin of paragraph
+  Point eop( const Point & p ) const;	// end of paragraph
 
   int set_to_matching_delimiter( Point & p, bool forward = true,
-                                 const bool force = false ) const throw();
-  bool find_text( Point & p, const std::string & text, const bool icase = false ) const throw();
-  bool rfind_text( Point & p, const std::string & text, const bool icase = false ) const throw();
+                                 const bool force = false ) const;
+  bool find_text( Point & p, const std::string & text, const bool icase = false ) const;
+  bool rfind_text( Point & p, const std::string & text, const bool icase = false ) const;
 
-  Point to_cursor( const Point & pointer ) const throw();
-  Point to_pointer( const Point & cursor, Point & pcursor ) const throw();
+  Point to_cursor( const Point & pointer ) const;
+  Point to_pointer( const Point & cursor, Point & pcursor ) const;
   int to_string( const Point & p1, const Point & p2, std::string & text ) const;
+  int to_string( const Point & p1, const int size, std::string & text ) const;
   };
 
 
 class Buffer : public Basic_buffer
   {
-  mutable bool _modified, saved;
-  mutable std::string _name;
+  static const std::string unnamed;
+  mutable bool backuped, modified_, saved;
+  mutable std::string name_;
 
   struct Change_history
     {
@@ -158,7 +165,7 @@ class Buffer : public Basic_buffer
         : before( pb ), after( pa ), modified( m ) {}
       void add_atom( const Atom & a ) { atom_vector.push_back( a ); }
       bool append_record( const Basic_buffer & buffer, Record & r );
-      void delete_atoms() throw();
+      void delete_atoms();
       };
 
     int index;			// Index of record to be undone
@@ -168,10 +175,10 @@ class Buffer : public Basic_buffer
 
     Change_history()
       : index( -1 ), appendable( false ), force_append( false ) {}
-    ~Change_history() throw();
+    ~Change_history();
 
     void add_record( const Basic_buffer & buffer, Record & r );
-    int records() const throw() { return record_vector.size(); }
+    int records() const { return record_vector.size(); }
     };
 
   Change_history change_history;
@@ -179,40 +186,42 @@ class Buffer : public Basic_buffer
 public:
   struct Error
     {
-    const char * s;
-    Error( const char * p ) { s = p; }
+    const char * const msg;
+    explicit Error( const char * const s ) : msg( s ) {}
     };
 
   class Options
     {
-    int _lmargin, _rmargin;
+    int lmargin_, rmargin_;
   public:
     bool auto_indent, overwrite, read_only, word_wrap;
 
-    Options() throw()
-      : _lmargin( 0 ), _rmargin( 71 ), auto_indent( false ),
+    Options()
+      : lmargin_( 0 ), rmargin_( 71 ), auto_indent( false ),
       overwrite( false ), read_only( false ), word_wrap( false ) {}
 
-    void reset() throw() { *this = Options(); }
-    int lmargin() const throw() { return _lmargin; }
-    int rmargin() const throw() { return _rmargin; }
-    bool set_lmargin( const int l ) throw();
-    bool set_rmargin( const int r ) throw();
+    void reset() { *this = Options(); }
+    int lmargin() const { return lmargin_; }
+    int rmargin() const { return rmargin_; }
+    bool set_lmargin( const int l );
+    bool set_rmargin( const int r );
     };
 
   Options options;
   Point mark[10];		// user bookmarks
   bool crlf;
 
-  Buffer( const Options & opts, const std::string * namep = 0 );
-  bool save( const std::string * namep = 0, bool set_name = true, bool append = false,
-             Point p1 = Point(), Point p2 = Point() ) const;
+  explicit Buffer( const Options & opts, const std::string * const namep = 0 );
+  int set_marks();		// set initial user bookmarks
+  bool save( const std::string * namep = 0, bool set_name = true,
+             bool append = false, Point p1 = Point(), Point p2 = Point() ) const;
 
-  const std::string & character_info( const Point & p ) const throw();
-  bool modified() const throw() { return _modified; }
-  void name( const std::string & n ) { _name = n; }
-  const std::string & name() const throw() { return _name; }
-  const std::string & uname() const throw();
+  long character_offset( const Point & p, int & percent ) const;
+  bool modified() const { return modified_; }
+  void name( const std::string & n ) { name_ = n; }
+  const std::string & name() const { return name_; }
+  const std::string & nename() const			// non-empty name
+    { if( name_.size() ) return name_; else return unnamed; }
 
        // insert block b.[p1,p2) at p and move next
   bool pputb( Point & p, const Basic_buffer & b, const Point & p1,
@@ -224,7 +233,7 @@ public:
   bool pdelb( const Point & p1, const Point & p2, const int save_change = 1 );
        // delete char at 'p', or at '--p' if back is true
   bool pdelc( Point & p, const bool back = false );
-  bool pdell( Point & p ) throw();	// delete line and set 'p' to bol()
+  bool pdell( Point & p );	// delete line and set 'p' to bol()
 
        // replace block at [p1,p2) with b.[bp1,bp2) and move p2 next
   bool replace( const Point & p1, Point & p2, const Basic_buffer & b,
@@ -232,7 +241,7 @@ public:
 
   bool undo( Point & p );
   bool redo( Point & p );
-  void force_append() const throw() { change_history.force_append = true; }
-  void reset_appendable() const throw()
+  void force_append() const { change_history.force_append = true; }
+  void reset_appendable() const
     { change_history.appendable = change_history.force_append = false; }
   };
