@@ -1,5 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,7 +32,8 @@
 
 namespace Bufhandle_vector {
 
-int _first;				// index of first element
+int first_;				// index of first element
+int last_visited_;			// absolute index for last_visited
 std::vector< Buffer_handle > data;
 
 
@@ -39,22 +41,24 @@ std::vector< Buffer_handle > data;
 //
 void emergency_save()
   {
-  if( handles() == 0 ) return;
-  int i;
-  for( i = 0; i < handles(); ++i )
+  if( !handles() ) return;
+  int i = 0;
+  for( ; i < handles(); ++i )
     if( data[i].buffer().modified() ) break;
   if( i >= handles() ) return;
 
   const std::string dead_name( "DEADMOE" );
   RC::editor_options().backup = false;
-  FILE *f = std::fopen( dead_name.c_str(), "a" );
-  time_t dummy = time( 0 );
-  std::fprintf( f, "\n*** Modified files in Moe when it aborted on %s", ctime( &dummy ) );
+  FILE * f = std::fopen( dead_name.c_str(), "a" );
+  if( !f ) return;
+  const time_t t = std::time( 0 );
+  std::fprintf( f, "\n*** Modified files in Moe when it aborted on %s",
+                std::ctime( &t ) );
   std::fflush( f );
   for( i = 0; i < handles(); ++i )
     if( data[i].buffer().modified() )
       {
-      std::fprintf( f, "\n*** File '%s'\n", data[i].buffer().uname().c_str() );
+      std::fprintf( f, "\n*** File '%s'\n", data[i].buffer().nename().c_str() );
       std::fflush( f );
       data[i].buffer().save( &dead_name, false, true );
       }
@@ -64,11 +68,11 @@ void emergency_save()
 } // end namespace Bufhandle_vector
 
 
-void Bufhandle_vector::init() throw()
-  { _first = 0; std::atexit( emergency_save ); }
+void Bufhandle_vector::init()
+  { first_ = 0; last_visited_ = 0; std::atexit( emergency_save ); }
 
 
-void Bufhandle_vector::save_all_named() throw()
+void Bufhandle_vector::save_all_named()
   {
   int count = 0;
   bool done = true;
@@ -82,7 +86,7 @@ void Bufhandle_vector::save_all_named() throw()
       catch( Buffer::Error ) { done = false; }
       }
     }
-  const char * s = ( done ? "" : "Only " );
+  const char * const s = done ? "" : "Only ";
   char buf[80];
   snprintf( buf, sizeof buf, "%s%d named buffer(s) saved", s, count );
   Screen::show_status_lines();
@@ -90,14 +94,14 @@ void Bufhandle_vector::save_all_named() throw()
   }
 
 
-void Bufhandle_vector::show_status() throw()
+void Bufhandle_vector::show_status()
   {
   int mcount = 0, ucount = 0;
 
   for( int i = 0; i < handles(); ++i )
     {
     if( data[i].buffer().modified() ) ++mcount;
-    if( !data[i].buffer().name().size() ) ++ucount;
+    if( data[i].buffer().name().empty() ) ++ucount;
     }
 
   char buf[80];
@@ -109,22 +113,22 @@ void Bufhandle_vector::show_status() throw()
   }
 
 
-int Bufhandle_vector::absolute_index( const int i ) throw()
-  { return ( i + _first ) % handles(); }
+int Bufhandle_vector::absolute_index( const int i )
+  { return ( i + first_ ) % handles(); }
 
 
-int Bufhandle_vector::circular_index( const int ai ) throw()
-  { return ( ai + handles() - _first ) % handles(); }
+int Bufhandle_vector::circular_index( const int ai )
+  { return ( ai + handles() - first_ ) % handles(); }
 
 
-Buffer_handle & Bufhandle_vector::handle( const int i ) throw()
+Buffer_handle & Bufhandle_vector::handle( const int i )
   { return data[absolute_index(i)]; }
 
 
-int Bufhandle_vector::handles() throw() { return data.size(); }
+int Bufhandle_vector::handles() { return data.size(); }
 
 
-int Bufhandle_vector::handles( const Buffer & buffer ) throw()
+int Bufhandle_vector::handles( const Buffer & buffer )
   {
   int c = 0;
   for( int i = 0; i < handles(); ++i ) if( &buffer == &data[i].buffer() ) ++c;
@@ -132,30 +136,29 @@ int Bufhandle_vector::handles( const Buffer & buffer ) throw()
   }
 
 
-int Bufhandle_vector::last_visited( const int i ) throw()
+int Bufhandle_vector::last_visited( const int i )
   {
-  static int ai = 0;
-  const int ci = circular_index( ai );
-  ai = absolute_index( i );
+  const int ci = circular_index( last_visited_ );
+  last_visited_ = absolute_index( i );
   return ci;
   }
 
 
-void Bufhandle_vector::next() throw()
-  { ++_first; _first %= handles(); }
+void Bufhandle_vector::next()
+  { ++first_; first_ %= handles(); }
 
 
-void Bufhandle_vector::prev() throw()
-  { --_first; _first += handles(); _first %= handles(); }
+void Bufhandle_vector::prev()
+  { --first_; first_ += handles(); first_ %= handles(); }
 
 
-void Bufhandle_vector::set_first( const int ai ) throw()
-  { _first = ai % handles(); }
+void Bufhandle_vector::set_first( const int ai )
+  { first_ = ai % handles(); }
 
 
 int Bufhandle_vector::find_or_add_handle( const Buffer::Options & opts,
-                                          const std::string * namep,
-                                          int i, const int line )
+                                          const std::string * const namep,
+                                          int i, const int line, const int col )
   {
   if( namep && namep->size() )
     for( int j = 0; j < handles(); ++j )
@@ -169,30 +172,34 @@ int Bufhandle_vector::find_or_add_handle( const Buffer::Options & opts,
   else
     {
     i = absolute_index( i );
-    if( i == 0 ) i = handles(); else if( i <= _first ) ++_first;
+    if( i == 0 ) i = handles();
+    else { if( i <= first_ ) ++first_;
+           if( i <= last_visited_ ) ++last_visited_; }
     }
-  data.insert( data.begin() + i, Buffer_handle( *bufp, line ) );
+  data.insert( data.begin() + i, Buffer_handle( *bufp, line, col ) );
   return circular_index( i );
   }
 
 
      // Add empty buffer if no files given or stdin is not a tty
-void Bufhandle_vector::add_handle_if_pending_input_or_empty( const int line )
+void Bufhandle_vector::add_handle_if_pending_input_or_empty( const int line,
+                                                             const int col )
   {
   if( !handles() || !isatty( fileno( stdin ) ) )
-    find_or_add_handle( RC::default_buffer_options(), 0, -1, line );
+    find_or_add_handle( RC::default_buffer_options(), 0, -1, line, col );
   }
 
 
 void Bufhandle_vector::duplicate_handle( int i )
   {
   i = absolute_index( i );
-  if( i < _first ) ++_first;
+  if( i < first_ ) ++first_;
+  if( i < last_visited_ ) ++last_visited_;
   data.insert( data.begin() + i + 1, data[i] );
   }
 
 
-bool Bufhandle_vector::delete_handle( int i ) throw()
+bool Bufhandle_vector::delete_handle( int i )
   {
   if( i < 0 || i >= handles() ) return false;
   else i = absolute_index( i );
@@ -206,6 +213,8 @@ bool Bufhandle_vector::delete_handle( int i ) throw()
     delete &buffer;
     }
   data.erase( data.begin() + i );
-  if( i <= _first && handles() ) prev();
+  if( i <= first_ && handles() ) prev();
+  if( i <= last_visited_ && handles() )
+    { --last_visited_; last_visited_ += handles(); last_visited_ %= handles(); }
   return true;
   }
