@@ -1,6 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
-    Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -102,68 +102,81 @@ int base64_value( const unsigned char ch )
 
 int map_to_byte( const int code )
   {
-  enum {
-       CGBREVE = 0x011E,	// latin capital letter g with breve
-       SGBREVE = 0x011F,	// latin small letter g with breve
-       CIDOT   = 0x0130,	// latin capital letter i with dot above
-       SINODOT = 0x0131,	// latin small letter i dotless
-       CSCEDI  = 0x015E,	// latin capital letter s with cedilla
-       SSCEDI  = 0x015F,	// latin small letter s with cedilla
-       CSCARON = 0x0160,	// latin capital letter s with caron
-       SSCARON = 0x0161,	// latin small letter s with caron
-       CZCARON = 0x017D,	// latin capital letter z with caron
-       SZCARON = 0x017E,	// latin small letter z with caron
-       EURO    = 0x20AC 	// symbole euro
-       };
   if( code < 0 ) return -1;
   if( code < 256 ) return code;
   switch( code )
     {
-    case CGBREVE: return 0xD0;
-    case SGBREVE: return 0xF0;
-    case CIDOT  : return 0xDD;
-    case SINODOT: return 0xFD;
-    case CSCEDI : return 0xDE;
-    case SSCEDI : return 0xFE;
-    case CSCARON: return 0xA6;
-    case SSCARON: return 0xA8;
-    case CZCARON: return 0xB4;
-    case SZCARON: return 0xB8;
-    case EURO   : return 0xA4;
-    default     : return -1;
+    case 0x0160: return 0xA6;	// latin capital letter s with caron
+    case 0x0161: return 0xA8;	// latin small letter s with caron
+    case 0x0178: return 0xBE;	// latin capital letter y with diaeresis
+    case 0x017D: return 0xB4;	// latin capital letter z with caron
+    case 0x017E: return 0xB8;	// latin small letter z with caron
+    case 0x2022: return 0xB7;	// bullet
+    case 0x2024: return '.';
+    case 0x2035: return '`';
+    case 0x2039: return '<';
+    case 0x203A: return '>';
+    case 0x2044: return '/';
+    case 0x204A: return '&';
+    case 0x204B: return 0xB6;	// reversed pilcrow sign
+    case 0x204E: return '*';
+    case 0x204F: return ';';
+    case 0x2052: return '%';
+    case 0x2053: return '~';
+    case 0x20AC: return 0xA4;	// symbole euro
     }
+  if( ( code >= 0x2000 && code <= 0x200B ) || code == 0x202F || code == 0x205F )
+    return ' ';
+  if( ( code >= 0x2010 && code <= 0x2013 ) || code == 0x2043 ) return '-';
+  if( ( code >= 0x2018 && code <= 0x201B ) || code == 0x2032 ) return '\'';
+  if( ( code >= 0x201C && code <= 0x201F ) || code == 0x2033 || code == 0x2036 )
+    return '"';
+  return -1;
   }
 
 
-// 'seq' contains an utf8 (possibly) multibyte character sequence.
-// Returns the corresponding code and, in *lenp, the characters read.
-// Returns -1 if error.
-//
-int utf8_to_ucs( const std::string & seq, const unsigned i, int *lenp )
+const char * map_to_string( const int code )
   {
-  if( i >= seq.size() ) return -1;
-  int len = 1;
-  unsigned char first = seq[i];
-  if( first < 128 )					// plain ascii
-    { if( lenp ) { *lenp = len; } return first; }
-  if( first < 192 || first > 253 ) return -1;		// invalid byte
-
-  ++len;
-  unsigned char bit = 0x20, mask = 0x1F;
-  while( first & bit ) { ++len; bit >>= 1; mask >>= 1; }
-  int code = first & mask;
-
-  for( int j = 1; j < len; ++j )
+  switch( code )
     {
-    unsigned char next = seq[i+j];
-    if( ( next & 0xC0 ) != 0x80 ) return -1;		// invalid byte
-    code = ( code << 6 ) | ( next & 0x3F );
+    case 0x0152: return "OE";
+    case 0x0153: return "oe";
+    case 0x2014:
+    case 0x2015: return "--";
+    case 0x2025: return "..";
+    case 0x2026: return "...";
+    case 0x2034: return "'''";
+    case 0x2037: return "```";
+    case 0x203C: return "!!";
+    case 0x2047: return "??";
+    case 0x2048: return "?!";
+    case 0x2049: return "!?";
+    case 0x2057: return "''''";
     }
+  return 0;
+  }
 
-  if( code < 0x80 || ( len > 2 && code < 0x800 << ( ( len - 3 ) * 5 ) ) )
-    return -1;						// no minimum length
-  if( lenp ) *lenp = len;
-  return code;
+
+const char * ucs_to_utf8( const int code )
+  {
+  static char s[7];
+
+  if( code < 0 || code > 0x7FFFFFFF ) { s[0] = 0; return s; } // invalid code
+  if( code < 128 ) { s[0] = code; s[1] = 0; return s; }       // plain ascii
+
+  int i, mask;
+  if( code < 0x800 ) { i = 2; mask = 0xC0; }		// 110X XXXX
+  else if( code < 0x10000 ) { i = 3; mask = 0xE0; }	// 1110 XXXX
+  else if( code < 0x200000 ) { i = 4; mask = 0xF0; }	// 1111 0XXX
+  else if( code < 0x4000000 ) { i = 5; mask = 0xF8; }	// 1111 10XX
+  else { i = 6; mask = 0xFC; }				// 1111 110X
+
+  s[i] = 0; --i;
+  int d = 0;
+  for( ; i > 0; --i, d+=6 )
+    s[i] = 0x80 | ( ( code >> d ) & 0x3F );		// 10XX XXXX
+  s[0] = mask | ( code >> d );
+  return s;
   }
 
 } // end namespace Encoding
@@ -180,12 +193,12 @@ void Encoding::base64_encode( const std::string & in, std::string & out )
     const bool s1 = ( i + 1 < in.size() );
     const bool s2 = ( i + 2 < in.size() );
     const unsigned char c0 = in[i];
-    const unsigned char c1 = ( s1 ? in[i+1] : 0 );
-    const unsigned char c2 = ( s2 ? in[i+2] : 0 );
+    const unsigned char c1 = s1 ? in[i+1] : 0;
+    const unsigned char c2 = s2 ? in[i+2] : 0;
     out += b64str[(c0 >> 2) & 0x3f];
     out += b64str[((c0 << 4) + (c1 >> 4)) & 0x3f];
-    out += ( s1 ? b64str[((c1 << 2) + (c2 >> 6)) & 0x3f] : '=' );
-    out += ( s2 ? b64str[c2 & 0x3f] : '=' );
+    out += s1 ? b64str[((c1 << 2) + (c2 >> 6)) & 0x3f] : '=';
+    out += s2 ? b64str[c2 & 0x3f] : '=';
     }
   }
 
@@ -193,45 +206,53 @@ void Encoding::base64_encode( const std::string & in, std::string & out )
 /* Decode base64 encoded input string 'in' to output string 'out'.
    Return true if decoding was successful, i.e. if the input was valid
    base64 data. Note that as soon as any invalid character is
-   encountered, decoding is stopped and false is returned. This means
-   that you must remove any line terminators from the input string
-   before calling this function.
+   encountered, decoding is stopped, the index of the invalid 4-byte
+   group is stored in *idxp, and false is returned. This means that you
+   must remove any line terminators from the input string before calling
+   this function.
 */
-bool Encoding::base64_decode( const std::string & in, std::string & out )
+bool Encoding::base64_decode( const std::string & in, std::string & out,
+                              int * const idxp )
   {
   unsigned i;
   out.clear();
 
-  for( i = 0; i + 1 < in.size(); i += 4 )
+  for( i = 0; i + 3 < in.size(); i += 4 )
     {
     const int i0 = base64_value( in[i] );
     const int i1 = base64_value( in[i+1] );
     if( i0 < 0 || i1 < 0 ) break;
     out += ( i0 << 2 ) | ( i1 >> 4 );
-    if( i + 2 >= in.size() ) break;
     if( in[i+2] == '=' )
       { if( i + 4 != in.size() || in[i+3] != '=' ) break; }
     else
       {
       const int i2 = base64_value( in[i+2] );
       if( i2 < 0 ) break;
-      out += ( ( ( i1 << 4 ) & 0xf0 ) | ( i2 >> 2 ) );
-      if( i + 3 >= in.size() ) break;
+      out += ( ( i1 << 4 ) & 0xf0 ) | ( i2 >> 2 );
       if( in[i+3] == '=')
         { if( i + 4 != in.size() ) break; }
       else
 	{
         const int i3 = base64_value( in[i+3] );
         if( i3 < 0 ) break;
-        out += ( ( ( i2 << 6 ) & 0xc0 ) | i3 );
+        out += ( ( i2 << 6 ) & 0xc0 ) | i3;
 	}
       }
     }
+  if( idxp ) *idxp = i;
   return ( i == in.size() );
   }
 
 
-bool Encoding::quoted_printable_decode( const std::string & in, std::string & out )
+/* Decode quoted-printable encoded input string 'in' to output string 'out'.
+   Return 0 if decoding was successful, i.e. if the input was valid
+   quoted-printable data. Note that as soon as any invalid character is
+   encountered, decoding is stopped, the index of the invalid character
+   is stored in *idxp, and false is returned.
+*/
+bool Encoding::quoted_printable_decode( const std::string & in,
+                                        std::string & out, int * const idxp )
   {
   unsigned i;
   out.clear();
@@ -252,10 +273,11 @@ bool Encoding::quoted_printable_decode( const std::string & in, std::string & ou
       const int i2 = ISO_8859::xtoi( ch2 );
       if( i1 < 0 || i2 < 0 || std::islower( ch1 ) || std::islower( ch2 ) )
         break;
-      out += ( ( i1 << 4 ) + i2 );
+      out += ( i1 << 4 ) + i2;
       i += 2;
       }
     }
+  if( idxp ) *idxp = i;
   return ( i == in.size() );
   }
 
@@ -329,22 +351,80 @@ unsigned char Encoding::rot47( const unsigned char ch )
   }
 
 
-/* Decode utf8 encoded input string 'in' to output string 'out'.
-   Return true if decoding was successful, i.e. if the input was valid
-   utf8 data in the iso-8859-[1|15] range. Note that as soon as any
-   invalid character is encountered, decoding is stopped and false is
-   returned.
+/* Encode ISO-8859-15 encoded input string 'in' to output string 'out'.
+   Return false if no encoding is needed, i.e. if the input is already
+   valid UTF-8 data.
 */
-bool Encoding::utf8_decode( const std::string & in, std::string & out )
+bool Encoding::utf8_encode( const std::string & in, std::string & out )
   {
-  unsigned i;
   out.clear();
-
-  for( i = 0; i < in.size(); )
+  for( unsigned i = 0; i < in.size(); )
     {
-    int len, code = map_to_byte( utf8_to_ucs( in, i, &len ) );
-    if( code < 0 ) break;
-    out += code; i += len;
+    int len;
+    if( utf8_to_ucs( in, i, &len ) >= 0 ) { i += len; continue; }
+    for( i = 0; i < in.size(); ++i )
+      out += ucs_to_utf8( (unsigned char)in[i] );
+    return true;
     }
-  return ( i == in.size() );
+  return false;
+  }
+
+
+// 'seq' contains an UTF-8 (possibly) multibyte character sequence.
+// Returns the corresponding code and, in *lenp, the characters read.
+// Returns -1 if error.
+//
+int Encoding::utf8_to_ucs( const std::string & seq, const unsigned i,
+                           int * const lenp )
+  {
+  if( i >= seq.size() ) return -1;
+  int len = 1;
+  unsigned char first = seq[i];
+  if( first < 128 )					// plain ascii
+    { if( lenp ) { *lenp = len; } return first; }
+  if( first < 192 || first > 253 ) return -1;		// invalid byte
+
+  ++len;
+  unsigned char bit = 0x20, mask = 0x1F;
+  while( first & bit ) { ++len; bit >>= 1; mask >>= 1; }
+  int code = first & mask;
+
+  for( int j = 1; j < len; ++j )
+    {
+    unsigned char next = seq[i+j];
+    if( ( next & 0xC0 ) != 0x80 ) return -1;		// invalid byte
+    code = ( code << 6 ) | ( next & 0x3F );
+    }
+
+  if( code < 0x80 || ( len > 2 && code < 0x800 << ( ( len - 3 ) * 5 ) ) )
+    return -1;						// no minimum length
+  if( lenp ) *lenp = len;
+  return code;
+  }
+
+
+/* Decode UTF-8 encoded input string 'in' to output string 'out'.
+   Return 0 if decoding was successful, i.e. if the input was valid
+   UTF-8 data in the ISO-8859-[1|15] range. Note that as soon as any
+   invalid character is encountered, decoding is stopped, the index of
+   the invalid character is stored in *idxp, and:
+     if invalid UTF-8 data is found then -1 is returned,
+     else the UCS code of the first character out of range is returned.
+*/
+int Encoding::utf8_decode( const std::string & in, std::string & out,
+                           int * const idxp )
+  {
+  out.clear();
+  for( unsigned i = 0; i < in.size(); )
+    {
+    int len;
+    const int code = utf8_to_ucs( in, i, &len );
+    if( code < 0 ) { if( idxp ) { *idxp = i; } return -1; }
+    const int ch = map_to_byte( code );
+    if( ch >= 0 ) { out += ch; i += len; continue; }
+    const char * const str = map_to_string( code );
+    if( !str ) { if( idxp ) { *idxp = i; } return code; }
+    out += str; i += len;
+    }
+  return 0;
   }
