@@ -1,5 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,16 +18,20 @@
 
 namespace Encoding {
 
-void base64_encode( const std::string & in, std::string & out ) throw();
+void base64_encode( const std::string & in, std::string & out );
 
-bool base64_decode( const std::string & in, std::string & out ) throw();
+bool base64_decode( const std::string & in, std::string & out,
+                    int * const idxp );
 
-bool quoted_printable_decode( const std::string & in, std::string & out ) throw();
+bool quoted_printable_decode( const std::string & in, std::string & out,
+                              int * const idxp );
 
-unsigned char rot13( const unsigned char ch ) throw();
+unsigned char rot13( const unsigned char ch );
 
-unsigned char rot47( const unsigned char ch ) throw();
+unsigned char rot47( const unsigned char ch );
 
-bool utf8_decode( const std::string & in, std::string & out ) throw();
+bool utf8_encode( const std::string & in, std::string & out );
+int utf8_to_ucs( const std::string & seq, const unsigned i, int * const lenp );
+int utf8_decode( const std::string & in, std::string & out, int * const idxp );
 
 } // end namespace Encoding
