@@ -1,6 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
-    Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -44,6 +44,7 @@ enum {
      CIACUTE = 0xCD,	// latin capital letter i with acute
      CICIRCU = 0xCE,	// latin capital letter i with circumflex
      CIDIAER = 0xCF,	// latin capital letter i with diaeresis
+     CETH    = 0xD0,	// latin capital letter eth (icelandic)
      CNTILDE = 0xD1,	// latin capital letter n with tilde
      COGRAVE = 0xD2,	// latin capital letter o with grave
      COACUTE = 0xD3,	// latin capital letter o with acute
@@ -56,6 +57,7 @@ enum {
      CUCIRCU = 0xDB,	// latin capital letter u with circumflex
      CUDIAER = 0xDC,	// latin capital letter u with diaeresis
      CYACUTE = 0xDD,	// latin capital letter y with acute
+     CTHORN  = 0xDE,	// latin capital letter thorn (icelandic)
      SAGRAVE = 0xE0,	// latin small letter a with grave
      SAACUTE = 0xE1,	// latin small letter a with acute
      SACIRCU = 0xE2,	// latin small letter a with circumflex
@@ -71,6 +73,7 @@ enum {
      SIACUTE = 0xED,	// latin small letter i with acute
      SICIRCU = 0xEE,	// latin small letter i with circumflex
      SIDIAER = 0xEF,	// latin small letter i with diaeresis
+     SETH    = 0xF0,	// latin small letter eth (icelandic)
      SNTILDE = 0xF1,	// latin small letter n with tilde
      SOGRAVE = 0xF2,	// latin small letter o with grave
      SOACUTE = 0xF3,	// latin small letter o with acute
@@ -83,6 +86,7 @@ enum {
      SUCIRCU = 0xFB,	// latin small letter u with circumflex
      SUDIAER = 0xFC,	// latin small letter u with diaeresis
      SYACUTE = 0xFD,	// latin small letter y with acute
+     STHORN  = 0xFE,	// latin small letter thorn (icelandic)
      SYDIAER = 0xFF	// latin small letter y with diaeresis
      };
 
@@ -97,7 +101,7 @@ int ISO_8859::escape( const std::string & seq, const unsigned i, int *lenp )
   {
   if( i >= seq.size() ) return -1;
   int len = 1;
-  unsigned char ch = seq[i];
+  unsigned ch = seq[i];
 
   switch( ch )
     {
@@ -109,14 +113,13 @@ int ISO_8859::escape( const std::string & seq, const unsigned i, int *lenp )
     case 'r': ch = '\r'; break;
     case 't': ch = '\t'; break;
     case 'v': ch = '\v'; break;
-    case '0':
-    case '1':
-    case '2':
-    case '3': if( i + 2 >= seq.size() || !ISO_8859::isodigit( seq[i+1] ) ||
-                  !ISO_8859::isodigit( seq[i+2] ) ) return -1;
-              ch = ((ch-'0') << 6) + ((seq[i+1]-'0') << 3) + (seq[i+2]-'0');
-              len = 3;
-              break;
+    case '0': case '1': case '2': case '3':
+    case '4': case '5': case '6': case '7':
+      ch -= '0';
+      while( len < 3 && i + len < seq.size() && ISO_8859::isodigit( seq[i+len] ) )
+        { ch <<= 3; ch += seq[i+len] - '0'; ++len; }
+      if( ch > 255 ) return -1;
+      break;
     case 'x':
     case 'X': if( i + 2 >= seq.size() || ISO_8859::xtoi( seq[i+1] ) < 0 ||
                   ISO_8859::xtoi( seq[i+2] ) < 0 ) return -1;
@@ -194,6 +197,7 @@ bool ISO_8859::islower( const unsigned char ch )
     case SIACUTE:
     case SICIRCU:
     case SIDIAER:
+    case SETH   :
     case SNTILDE:
     case SOGRAVE:
     case SOACUTE:
@@ -206,6 +210,7 @@ bool ISO_8859::islower( const unsigned char ch )
     case SUCIRCU:
     case SUDIAER:
     case SYACUTE:
+    case STHORN :
     case SYDIAER: return true;
     default:      return false;
     }
@@ -234,6 +239,7 @@ bool ISO_8859::isupper( const unsigned char ch )
     case CIACUTE:
     case CICIRCU:
     case CIDIAER:
+    case CETH   :
     case CNTILDE:
     case COGRAVE:
     case COACUTE:
@@ -245,7 +251,8 @@ bool ISO_8859::isupper( const unsigned char ch )
     case CUACUTE:
     case CUCIRCU:
     case CUDIAER:
-    case CYACUTE: return true;
+    case CYACUTE:
+    case CTHORN : return true;
     default:      return false;
     }
   }
@@ -263,7 +270,7 @@ bool ISO_8859::isalnum_( const unsigned char ch )
   }
 
 
-bool ISO_8859::isodigit( const unsigned char ch )	// tests if 'ch' is a octal digit
+bool ISO_8859::isodigit( const unsigned char ch )  // is 'ch' an octal digit?
   {
   switch( ch )
     {
@@ -330,6 +337,7 @@ unsigned char ISO_8859::tolower( const unsigned char ch )
     case CIACUTE: return SIACUTE;
     case CICIRCU: return SICIRCU;
     case CIDIAER: return SIDIAER;
+    case CETH   : return SETH;
     case CNTILDE: return SNTILDE;
     case COGRAVE: return SOGRAVE;
     case COACUTE: return SOACUTE;
@@ -342,6 +350,7 @@ unsigned char ISO_8859::tolower( const unsigned char ch )
     case CUCIRCU: return SUCIRCU;
     case CUDIAER: return SUDIAER;
     case CYACUTE: return SYACUTE;
+    case CTHORN : return STHORN;
     default     : return ch;
     }
   }
@@ -369,6 +378,7 @@ unsigned char ISO_8859::toupper( const unsigned char ch )
     case SIACUTE: return CIACUTE;
     case SICIRCU: return CICIRCU;
     case SIDIAER: return CIDIAER;
+    case SETH   : return CETH;
     case SNTILDE: return CNTILDE;
     case SOGRAVE: return COGRAVE;
     case SOACUTE: return COACUTE;
@@ -381,6 +391,7 @@ unsigned char ISO_8859::toupper( const unsigned char ch )
     case SUCIRCU: return CUCIRCU;
     case SUDIAER: return CUDIAER;
     case SYACUTE: return CYACUTE;
+    case STHORN : return CTHORN;
     default     : return ch;
     }
   }
