@@ -1,6 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
-    Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,6 +38,8 @@ bool isalpha_( const unsigned char ch );
 bool isalnum_( const unsigned char ch );
 inline bool isblank_( const unsigned char ch )
   { return ( ch == ' ' || ch == '\t' ); }
+inline bool isspace( const unsigned char ch )
+  { return ( std::isspace( ch ) || ch == 0xA0 ); }
 
 inline bool iscntrl( const unsigned char ch )
   { return ( ch < 32 || ( ch >= 0x7f && ch < 0xA0 ) ); }
