@@ -1,6 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 /*
-    Return values: 0 for a normal exit, 1 for environmental problems
+    Exit status: 0 for a normal exit, 1 for environmental problems
     (file not found, invalid flags, I/O errors, etc), 2 to indicate a
     corrupt or invalid input file, 3 for an internal consistency error
     (eg, bug) which caused moe to panic.
@@ -42,58 +42,63 @@ namespace {
 const char * const     Program_name = "GNU Moe";
 const char * const     program_name = "moe";
 const char * const config_file_name = "moerc";
-const char * const     program_year = "2011";
+const char * const     program_year = "2014";
 const char * invocation_name = 0;
 
 
-void show_help() throw()
+void show_help()
   {
   std::printf( "%s - A powerful and user-friendly text editor.\n", Program_name );
-  std::printf( "\nUsage: %s [global-options] [[+line] filename [file-options]]...\n", invocation_name );
-  std::printf( "  `global-options' are a mix of editor-options and default file-options.\n");
-  std::printf( "  Most long option names have a negative form `--backup' `--no-backup'.\n" );
-  std::printf( "\nEditor options:\n" );
-  std::printf( "  -h, --help               display this help and exit\n" );
-  std::printf( "  -V, --version            output version information and exit\n" );
-  std::printf( "  -1, --orphan             put extra files in orphaned buffers\n" );
-  std::printf( "  -b, --backup             create backup files (default)\n" );
-  std::printf( "  -e, --exit-ask           make save-and-close ask always for confirmation\n" );
-  std::printf( "  -H, --smart-home         go home, then go to first non-blank character\n" );
-  std::printf( "  -i, --ignore-case        make search case insensitive by default\n" );
-  std::printf( "  -k, --keep-lines=<n>     number of lines to keep for PgUp/PgDn\n" );
-  std::printf( "  -m, --max-windows=<n>    max number of windows to show at once\n" );
-  std::printf( "  -n, --indent-step=<n>    number of spaces to add when indenting a block\n" );
-//  std::printf( "  -p, --beep               beep on errors and when cursor goes past extremes\n" );
-  std::printf( "  -s, --search-wrap        make search wrap at end of file\n" );
-  std::printf( "  -u, --auto-unmark        turn off highlighting after block copy or move\n" );
-  std::printf( "  -x, --rectangle          rectangular block mode\n" );
-  std::printf( "\nFile options:\n" );
-  std::printf( "  -a, --auto-indent        enable auto indent\n" );
-  std::printf( "  -l, --lmargin=<col>      set left margin\n" );
-  std::printf( "  -r, --rmargin=<col>      set right margin\n" );
-  std::printf( "  -o, --read-only          make buffer read-only\n" );
-  std::printf( "  -O, --overwrite          disable insert mode\n" );
-  std::printf( "  -w, --word-wrap          enable word wrapping\n" );
-  std::printf( "\nReport bugs to bug-moe@gnu.org\n" );
-  std::printf( "Moe home page: http://www.gnu.org/software/moe/moe.html\n" );
-  std::printf( "General help using GNU software: http://www.gnu.org/gethelp\n" );
+  std::printf( "\nUsage: %s [global_options] [[+line[,col]] file_name [file_options]]...\n", invocation_name );
+  std::printf( "  'global_options' are a mix of editor options and default file_options.\n"
+               "  Most long option names have a negative form '--backup' '--no-backup'.\n"
+               "\nEditor options:\n"
+               "  -h, --help               display this help and exit\n"
+               "  -V, --version            output version information and exit\n"
+               "  -1, --orphan             put extra files in orphaned buffers\n"
+               "  -b, --backup             create backup files (default)\n"
+               "  -B, --no-backup          do not create backup files\n"
+               "  -e, --exit-ask           make save-and-close ask always for confirmation\n"
+               "  -H, --smart-home         go home, then go to first non-blank character\n"
+               "  -i, --ignore-case        make search case insensitive by default\n"
+               "  -k, --keep-lines=<n>     number of lines to keep for PgUp/PgDn\n"
+               "  -m, --max-windows=<n>    max number of windows to show at once\n"
+               "  -n, --indent-step=<n>    number of spaces to add when indenting a block\n"
+//               "  -p, --beep               beep on errors and when cursor goes past extremes\n"
+               "  -s, --search-wrap        make search wrap at end of file\n"
+               "  -u, --auto-unmark        turn off highlighting after block copy or move\n"
+               "  -x, --rectangle          rectangular block mode\n"
+               "\nFile options:\n"
+               "  -a, --auto-indent        enable auto indent\n"
+               "  -l, --lmargin=<col>      set left margin\n"
+               "  -r, --rmargin=<col>      set right margin\n"
+               "  -o, --read-only          make buffer read-only\n"
+               "  -O, --overwrite          disable insert mode\n"
+               "  -w, --word-wrap          enable word wrapping\n"
+               "\nExit status: 0 for a normal exit, 1 for environmental problems (file\n"
+               "not found, invalid flags, I/O errors, etc), 2 to indicate a corrupt or\n"
+               "invalid input file, 3 for an internal consistency error (eg, bug) which\n"
+               "caused moe to panic.\n"
+               "\nReport bugs to bug-moe@gnu.org\n"
+               "Moe home page: http://www.gnu.org/software/moe/moe.html\n"
+               "General help using GNU software: http://www.gnu.org/gethelp\n" );
   }
 
 
-void show_version() throw()
+void show_version()
   {
-  std::printf( "%s %s\n", Program_name, PROGVERSION );
+  std::printf( "GNU %s %s\n", program_name, PROGVERSION );
   std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
-  std::printf( "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n" );
-  std::printf( "This is free software: you are free to change and redistribute it.\n" );
-  std::printf( "There is NO WARRANTY, to the extent permitted by law.\n" );
+  std::printf( "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n"
+               "This is free software: you are free to change and redistribute it.\n"
+               "There is NO WARRANTY, to the extent permitted by law.\n" );
   }
 
 
-extern "C" void termination_handler( int ) throw() { std::exit( 1 ); }
+extern "C" void termination_handler( int ) { std::exit( 1 ); }
 
 
-int set_signal( int signum, void (*handler)(int) ) throw()
+int set_signal( const int signum, void (*handler)( int ) )
   {
   struct sigaction new_action;
 
@@ -104,7 +109,7 @@ int set_signal( int signum, void (*handler)(int) ) throw()
   }
 
 
-void restore_signals() throw()
+void restore_signals()
   {
   set_signal( SIGALRM, SIG_DFL );
   set_signal( SIGBUS, SIG_DFL );
@@ -120,7 +125,7 @@ void restore_signals() throw()
   }
 
 
-void set_signals() throw()
+void set_signals()
   {
   std::atexit( restore_signals );
   set_signal( SIGALRM, Screen::clock_handler );
@@ -137,7 +142,7 @@ void set_signals() throw()
   }
 
 
-bool use_it_anyway( std::string name ) throw()
+bool use_it_anyway( const std::string & name )
   {
   if( !isatty( fileno( stdin ) ) ) return false;
   std::fprintf( stderr, "There are errors in '%s'. Use it anyway (y/N)? ",
@@ -147,7 +152,7 @@ bool use_it_anyway( std::string name ) throw()
   }
 
 
-int process_config_file() throw()
+void process_config_file()
   {
   std::fprintf( stderr, "\n" );
   std::string name( RC::home_directory() );
@@ -155,45 +160,45 @@ int process_config_file() throw()
     {
     name += "/."; name += config_file_name;
     const int retval = RC::process_rcfile( name );
-    if( retval == 0 || ( retval == 2 && use_it_anyway( name ) ) ) return 0;
-    else RC::reset();
+    if( retval == 0 || ( retval == 2 && use_it_anyway( name ) ) ) return;
+    RC::reset();
     }
   name = SYSCONFDIR; name += '/'; name += config_file_name;
   const int retval = RC::process_rcfile( name );
-  if( retval == 0 || ( retval == 2 && use_it_anyway( name ) ) ) return 0;
-  else RC::reset();
-  std::fprintf( stderr, "Couldn't open '%s'\n", name.c_str() );
+  if( retval == 0 || ( retval == 2 && use_it_anyway( name ) ) ) return;
   RC::reset();
-  return 0;
+  std::fprintf( stderr, "Couldn't open '%s'\n", name.c_str() );
   }
 
 } // end namespace
 
 
-const std::string & RC::home_directory() throw()
+const std::string & RC::home_directory()
   {
   static bool first_time = true;
   static std::string home;
 
   if( first_time )
-    { first_time = false; const char *p = getenv( "HOME" ); if( p ) home = p; }
-
+    {
+    first_time = false;
+    const char * p = std::getenv( "HOME" ); if( p ) home = p;
+    }
   return home;
   }
 
 
 void RC::show_error( const char * const msg, const int errcode,
-                     const bool help ) throw()
+                     const bool help )
   {
   if( msg && msg[0] )
     {
     std::fprintf( stderr, "%s: %s", program_name, msg );
     if( errcode > 0 )
-      std::fprintf( stderr, ": %s", std::strerror( errcode ) );
+      std::fprintf( stderr, ": %s.", std::strerror( errcode ) );
     std::fprintf( stderr, "\n" );
     }
-  if( help && invocation_name && invocation_name[0] )
-    std::fprintf( stderr, "Try `%s --help' for more information.\n",
+  if( help )
+    std::fprintf( stderr, "Try '%s --help' for more information.\n",
                   invocation_name );
   }
 
@@ -212,15 +217,12 @@ int main( const int argc, const char * const argv[] )
 
   Bufhandle_vector::init();
 
-  retval = process_config_file();
-  if( !retval )
+  process_config_file();
+  retval = RC::process_options();
+  if( retval == 2 )
     {
-    retval = RC::process_options();
-    if( retval == 2 )
-      {
-      std::fprintf( stderr, "Press RETURN to continue, Ctrl-C to abort.\n" );
-      std::fgetc( stdin ); retval = 0;
-      }
+    std::fprintf( stderr, "Press RETURN to continue, Ctrl-C to abort.\n" );
+    std::fgetc( stdin ); retval = 0;
     }
   if( retval ) return retval;
 
@@ -229,7 +231,7 @@ int main( const int argc, const char * const argv[] )
 
   char buf[88];
   snprintf( buf, sizeof buf,
-    "@i ** My Own Editor v%s ** (iso-8859) ** Copyright (C) %s Antonio Diaz ** ",
+    "@i ** My Own Editor v%s ** (ISO-8859) ** Copyright (C) %s Antonio Diaz ** ",
     PROGVERSION, program_year );
   return Screen::user_loop( buf );
   }
