@@ -1,6 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ namespace Menu {
 
 void show( const std::vector< std::string > & name_vector, const int lines,
            const int columns, const int current, Point & cursor,
-           const bool indicator = false ) throw()
+           const bool indicator = false )
   {
   static int top_line = 0;
   const int entries = name_vector.size();
@@ -58,13 +58,13 @@ void show( const std::vector< std::string > & name_vector, const int lines,
       if( i == current )		// highlighted entry
         {
         i1 = s.size();
-        if( indicator ) ++i1;		// skip modified indicator
+        if( indicator ) ++i1;		// skip modified or crlf indicator
         s += name_vector[i];
         i2 = s.size();
         cursor = Point( line, std::min( i2, Screen::width() - 1 ) );
         }
       else s += name_vector[i];
-      const unsigned int col = std::min( Screen::width(), ( ( ( i % columns ) + 1 ) * ( Screen::width() + 2 ) ) / columns );
+      const unsigned col = std::min( Screen::width(), ( ( ( i % columns ) + 1 ) * ( Screen::width() + 2 ) ) / columns );
       if( (int)col < Screen::width() && s.size() > col - 2 ) s.resize( col - 2 );
       while( s.size() < col ) s += ' ';
       }
@@ -74,12 +74,12 @@ void show( const std::vector< std::string > & name_vector, const int lines,
 
 
 void append_common_prefix( std::string & name,
-                           const std::vector< std::string > & name_vector ) throw()
+                           const std::vector< std::string > & name_vector )
   {
-  for( unsigned int i = 0; i < name_vector[0].size(); ++i )
+  for( unsigned i = 0; i < name_vector[0].size(); ++i )
     {
     const char ch = name_vector[0][i];
-    for( unsigned int j = 1; j < name_vector.size(); ++j )
+    for( unsigned j = 1; j < name_vector.size(); ++j )
       if( i >= name_vector[j].size() || ch != name_vector[j][i] )
         return;
     name += ch;
@@ -87,25 +87,25 @@ void append_common_prefix( std::string & name,
   }
 
 
-bool append_slash_if_dir( std::string & name ) throw()
+bool append_slash_if_dir( std::string & name )
   {
   if( name.size() && name[name.size()-1] != '/' )
     {
     struct stat st;
-    if( !stat( name.c_str(), &st ) && S_ISDIR( st.st_mode ) )
+    if( stat( name.c_str(), &st ) == 0 && S_ISDIR( st.st_mode ) )
       { name += '/'; return true; }
     }
   return false;
   }
 
 
-void append_part_or_go_up( std::string & name, const std::string & new_part ) throw()
+void append_part_or_go_up( std::string & name, const std::string & new_part )
   {
   if( new_part == ".." )
     {
     if( name == "/" ) return;
     if( name == "//" ||
-        ( name.size() >= 4 && name.compare( name.size() - 4, 4, "/../" ) ) ||
+        ( name.size() >= 4 && name.compare( name.size() - 4, 4, "/../" ) != 0 ) ||
         ( name.size() == 3 && name != "../" ) )
       {
       int i = name.size() - 1; while( i > 0 && name[i-1] != '/' ) --i;
@@ -118,11 +118,11 @@ void append_part_or_go_up( std::string & name, const std::string & new_part ) th
 
 
 void split_filename( const std::string & name,
-                     std::string & dirname, std::string & basename ) throw()
+                     std::string & dirname, std::string & basename )
   {
-  unsigned int first = 0;		// first character of basename
+  unsigned first = 0;		// first character of basename
 
-  for( unsigned int i = name.size(); i > 0; --i )
+  for( unsigned i = name.size(); i > 0; --i )
     if( name[i-1] == '/' ) { first = i; break; }
 
   if( first > 0 ) dirname.assign( name, 0, first );
@@ -132,13 +132,6 @@ void split_filename( const std::string & name,
   else basename.clear();
   }
 
-
-bool valid_filename( const std::string & name ) throw()
-  {
-  const std::string & basename = my_basename( name );
-  return ( basename.size() && basename != "." && basename != ".." );
-  }
-
 } // end namespace Menu
 
 
@@ -148,13 +141,14 @@ int Menu::bufhandle_menu( int current, const int abort_key )
   {
   std::vector< std::string > name_vector;
 
+  const char * const indicator[4] = { " ", "*", "-", "+" };
   const int entries = Bufhandle_vector::handles();
   for( int i = 0; i < entries; ++i )
     {
     const int c = Bufhandle_vector::circular_index( i );
     const Buffer & buffer = Bufhandle_vector::handle( c ).buffer();
-    std::string tmp( " " ); tmp += buffer.nename();
-    if( buffer.modified() ) tmp[0] = '*';
+    const int idx = ( buffer.modified() ? 1 : 0 ) + ( buffer.crlf ? 2 : 0 );
+    std::string tmp( indicator[idx] ); tmp += buffer.nename();
     name_vector.push_back( tmp );
     }
 
@@ -175,7 +169,7 @@ int Menu::bufhandle_menu( int current, const int abort_key )
 
   Screen::save_lines_and_cursor( line, lines + 1 );
 
-  Screen::out_line( "Select buffer handle or 0-9 (^C to abort)", Screen::height() - 1 );
+  Screen::out_line( "Select buffer handle, F1-F10 or first char (^C to abort)", Screen::height() - 1 );
   current = Bufhandle_vector::absolute_index( current );
   int key = 0;
   while( key >= 0 )
@@ -188,33 +182,38 @@ int Menu::bufhandle_menu( int current, const int abort_key )
       valid_key = true;
       do key = Screen::wait_kbhit( line ); while( key < 0 );
       if( key == abort_key ) key = 3;
-      if( std::isdigit( key ) )
-        current = ( ( entries - 1 ) * ( key - '0' ) ) / 9;
+      const int fk = Screen::function_key( key );
+      if( fk >= 1 && fk <= 10 )
+        current = ( ( entries - 1 ) * ( fk - 1 ) ) / 9;
+      else if( key < 256 && ISO_8859::isgraph( key ) )
+        for( int i = current; ( i = ( i + 1 ) % entries ) != current; )
+          { if( (unsigned char)name_vector[i][1] == key )
+              { current = i; break; } }
       else switch( key )
         {
         case 3   : key = -1; break;		// ^C
         case ' ' :
         case '\n': key = -2; break;
         default: switch( Screen::convert_key( key ) )
-                   {
-                   case Screen::key_up   :
-                     if( current >= columns ) current -= columns; break;
-                   case Screen::key_down :
-                     if( current + columns < entries ) current += columns; break;
-                   case Screen::key_left : if( current > 0 ) --current; break;
-                   case Screen::key_right: if( current < entries - 1 ) ++current; break;
-                   case Screen::key_ppage:
-                     for( int i = 0; i < lines && current >= columns; ++i )
-                       current -= columns;
-                     break;
-                   case Screen::key_npage:
-                     for( int i = 0; i < lines && current + columns < entries; ++i )
-                       current += columns;
-                     break;
-                   case Screen::key_home : current = 0; break;
-                   case Screen::key_end  : current = entries - 1; break;
-                   default: valid_key = false;
-                   }
+          {
+          case Screen::key_up   :
+            if( current >= columns ) current -= columns; break;
+          case Screen::key_down :
+            if( current + columns < entries ) current += columns; break;
+          case Screen::key_left : if( current > 0 ) --current; break;
+          case Screen::key_right: if( current < entries - 1 ) ++current; break;
+          case Screen::key_ppage:
+            for( int i = 0; i < lines && current >= columns; ++i )
+              current -= columns;
+            break;
+          case Screen::key_npage:
+            for( int i = 0; i < lines && current + columns < entries; ++i )
+              current += columns;
+            break;
+          case Screen::key_home : current = 0; break;
+          case Screen::key_end  : current = entries - 1; break;
+          default: valid_key = false;
+          }
         }
       }
     }
@@ -224,7 +223,7 @@ int Menu::bufhandle_menu( int current, const int abort_key )
   }
 
 
-// 'name' contains the filename to be modified (may be empty).
+// 'name' contains the file name to be modified (may be empty).
 // Returns false if aborted.
 //
 bool Menu::file_menu( const std::string & prompt, std::string & name )
@@ -291,7 +290,7 @@ bool Menu::file_menu( const std::string & prompt, std::string & name )
         case '\n': key = -3; name = dirname;
                    append_part_or_go_up( name, name_vector[current] );
                    break;
-        default: if( key < 256 && ISO_8859::isprint( key ) )
+        default: if( key < 256 && ISO_8859::isgraph( key ) )
                    { name += key; key = -4; }
                  else switch( Screen::convert_key( key ) )
                    {
@@ -339,7 +338,7 @@ int Menu::get_filename( const std::string & prompt, std::string & name,
   }
 
 
-bool Menu::is_regular( const std::string & name, const bool quiet ) throw()
+bool Menu::is_regular( const std::string & name, const bool quiet )
   {
   if( name.size() && name[name.size()-1] != '/' )
     {
@@ -353,23 +352,23 @@ bool Menu::is_regular( const std::string & name, const bool quiet ) throw()
   }
 
 
-bool Menu::make_path( const std::string & name ) throw()
+bool Menu::make_path( const std::string & name )
   {
   std::string dirname, basename;
   split_filename( name, dirname, basename );
-  if( !basename.size() || basename == "." || basename == ".." ) return false;
+  if( basename.empty() || basename == "." || basename == ".." ) return false;
 
-  unsigned int index = 0;
+  unsigned index = 0;
   while( index < dirname.size() )
     {
     while( index < dirname.size() && dirname[index] == '/' ) ++index;
-    unsigned int first = index;
+    unsigned first = index;
     while( index < dirname.size() && dirname[index] != '/' ) ++index;
     if( first < index )
       {
       std::string partial( dirname, 0, index );
       struct stat st;
-      if( !stat( partial.c_str(), &st ) )
+      if( stat( partial.c_str(), &st ) == 0 )
         { if( !S_ISDIR( st.st_mode ) ) return false; }
       else if( mkdir( partial.c_str(), S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH ) )
         return false;
@@ -379,7 +378,7 @@ bool Menu::make_path( const std::string & name ) throw()
   }
 
 
-const std::string & Menu::my_basename( const std::string & name ) throw()
+const std::string & Menu::my_basename( const std::string & name )
   {
   static std::string basename;
   std::string dirname;
@@ -388,17 +387,18 @@ const std::string & Menu::my_basename( const std::string & name ) throw()
   }
 
 
-bool Menu::test_slash_filename( std::string & name ) throw()
+bool Menu::tilde_expansion( std::string & name )
   {
-  tilde_expansion( name );
-  return ( !name.size() ||
-           ( valid_filename( name ) && !append_slash_if_dir( name ) ) );
-  }
-
-
-bool Menu::tilde_expansion( std::string & name ) throw()
-  {
-  if( name.size() >= 2 && !name.compare( 0, 2, "~/" ) )
+  if( name.size() >= 2 && name.compare( 0, 2, "~/" ) == 0 )
     { name.replace( 0, 1, RC::home_directory() ); return true; }
   return false;
   }
+
+
+bool Menu::validate_filename( std::string & name )
+  {
+  tilde_expansion( name );
+  const std::string & basename = my_basename( name );
+  return ( basename.size() && basename != "." && basename != ".." &&
+           !append_slash_if_dir( name ) );
+  }
