@@ -1,6 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,30 +27,30 @@
 
 namespace Menu {
 
-const char *help_line[] = {
+const char * help_line[] = {
   " Help Screen: Keyboard Commands               PAGE_DN next",
   " @bF1@b Help           @bF5@b Prev window     @bF9@b Copy block     ^C  Close window",
   " @bF2@b Save file      @bF6@b Next window    @bF10@b Options menu   ^X  Save and close",
   " @bF3@b Load file      @bF7@b Undo           @bF11@b Buffers menu   ^QX Save all and exit",
   " @bF4@b Zoom window    @bF8@b Redo           @bF12@b Last visited   ^QC Exit without saving",
   " ",
-  " @bGO TO@b                     @bSEARCH@b                   @bMISC@b",
-  " ^[U Beg of file           ^F  Find                 Ins Insert/Overwrite",
-  " ^[V End of file           ^G  Find next            Tab Filename completion",
-  " ^[L Line                  ^W  Find word            ^B  Reformat paragraph",
-  " ^[C Column                ^OF Toggle direction     ^P  Control char",
-  " ^[O Offset                ^QU update (C)           ^Y  Delete line",
-  " ^[B Beg of block                                   ^OC Center line",
-  " ^[K End of block          @bBOOKMARKS@b                ^ON Change buffer name",
-  " ^[F Match pair ->         ^[ 0-9 Goto              ^OR Repaint screen",
-  " ^[G Match pair <-         ^K 0-9 Set               ^OS Split window",
-  " ^[D Center cursor                                  ^Q@bF2@b Save named buffers",
-  "                           @bINFO@b",
-  " @bSCROLL@b                    ^H  Show help",
-  " ^[W Up                    ^SC Show character code",
-  " ^[Z Down                  ^SG Show global status",
-  " ^[A Left                  ^SI Show character info",
-  " ^[S Right                 ^SV Show version",
+  " @bGO TO@b                  @bSEARCH@b                      @bMISC@b",
+  " ^[U Beg of file        ^F  Find                    Ins  Insert/Overwrite",
+  " ^[V End of file        ^G  Find next               Tab  File name completion",
+  " ^[L Line               ^W  Find word               ^B   Reformat paragraph",
+  " ^[C Column             ^OF Toggle direction        ^P   Control char",
+  " ^[O Offset             ^QU update (C)              ^Y   Delete line",
+  " ^[B Beg of block                                   ^OC  Center line",
+  " ^[K End of block       @bINFO@b                        ^ON  Change buffer name",
+  " ^[F Match pair ->      ^H   Show help              ^OR  Repaint screen",
+  " ^[G Match pair <-      ^[I  Show character info    ^OS  Split window",
+  " ^[D Center cursor      ^[T  Show UTF-8 code        ^Q@bF2@b Save named buffers",
+  "                        ^Sn  Show LE multibyte",
+  " @bSCROLL@b                 ^S@bFn@b Show BE multibyte      @bBOOKMARKS@b",
+  " ^[W Up                 ^SC  Show character code    ^[ 0-9 Goto",
+  " ^[Z Down               ^SG  Show global status     ^K 0-9 Set",
+  " ^[A Left               ^SV  Show version           ^KE    Extend",
+  " ^[S Right",
   "",
   " Help Screen: Block Commands     PAGE_DN next/PAGE_UP prev",
   " ",
@@ -58,12 +58,12 @@ const char *help_line[] = {
   " ^SPACE Mark begin/end     ^O1 To Base64          ^OD Remove duplicate lines ->",
   " ^KB Begin                 ^O3 To Rot-13          ^OE Remove duplicate lines <-",
   " ^KK End                   ^O4 To Rot-47",
-  " ^KC Copy",
-  " ^KM Move                  @bDECODE@b",
-  " ^KY Delete                ^O2 Base64",
-  " ^KR Read                  ^O6 Quoted-Printable",
-  " ^KW Write                 ^O8 UTF-8",
-  " ^KI Indent",
+  " ^KC Copy                  ^O7 To UTF-8",
+  " ^KM Move",
+  " ^KY Delete                @bDECODE@b",
+  " ^KR Read                  ^O2 Base64",
+  " ^KW Write                 ^O6 Quoted-Printable",
+  " ^KI Indent                ^O8 UTF-8",
   " ^KU Unindent",
   " ^OL To lowercase",
   " ^OU To uppercase",
@@ -105,10 +105,10 @@ const char *help_line[] = {
   " Help Screen: Replace            PAGE_DN next/PAGE_UP prev",
   " ",
   " Special replace sequences:",
-  "     \\&      replaced with text which matched search string",
-  "     \\c      same as \\& but matched text is capitalized",
-  "     \\l  \\u  same as \\& but matched text is lowercased / uppercased",
-  "     \\0 - 9  replaced with text which matched the Nth special search sequence",
+  "   \\&      replaced with text which matched search string",
+  "   \\c      same as \\& but matched text is capitalized",
+  "   \\l  \\u  same as \\& but matched text is lowercased / uppercased",
+  "   \\0 - 9  replaced with text which matched the Nth special search sequence",
   "",
   " Help Screen: Character Set                   PAGE_UP prev",
   " 32  \x20     56  \x38     80  \x50    104  \x68    160  \xA0    184  \xB8    208  \xD0    232  \xE8",
@@ -147,16 +147,16 @@ public:
     data.push_back( 0 );
     for( int i = 0; ; ++i )
       {
-      const char *p = help_line[i];
+      const char * p = help_line[i];
       if( !p || *p == 0 ) { data.push_back( i + 1 ); if( !p ) break; }
       }
     }
-  int operator[]( const int i ) const throw() { return data[i]; }
-  int size() const throw() { return data.size(); }
+  int operator[]( const int i ) const { return data[i]; }
+  int size() const { return data.size(); }
   } help_index;
 
 
-void show_help( int delta ) throw()
+void show_help( int delta )
   {
   static int current = 0;
 
@@ -199,8 +199,10 @@ void Menu::help_menu( const int abort_key1, const int abort_key2 )
       if( key == 3 || key == abort_key1 || key == abort_key2 )	// ^C
         { done = true; break; }
       key = Screen::convert_key( key );
-      if( key == Screen::key_npage ) { delta = +1; break; }
-      if( key == Screen::key_ppage ) { delta = -1; break; }
+      if( key == Screen::key_down || key == Screen::key_npage )
+        { delta = +1; break; }
+      if( key == Screen::key_up   || key == Screen::key_ppage )
+        { delta = -1; break; }
       }
     }
   Screen::restore_lines_and_cursor();
