@@ -1,6 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
-    Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "arg_parser.h"
 #include "buffer.h"
 #include "buffer_handle.h"
+#include "iso_8859.h"
 #include "menu.h"
 #include "rc.h"
 #include "regex.h"
@@ -299,7 +300,7 @@ void RC::apply_regex_options( Buffer & buffer )
 bool RC::issigned( const std::string & s )
   {
   unsigned i = 0;
-  while( i < s.size() && std::isspace( s[i] ) ) ++i;	// strip spaces
+  while( i < s.size() && ISO_8859::isspace( s[i] ) ) ++i;	// strip spaces
   return ( i < s.size() && ( s[i] == '+' || s[i] == '-' ) );
   }
 
@@ -307,11 +308,11 @@ bool RC::issigned( const std::string & s )
     // Returns the number of chars read, or 0 if error.
 int RC::parse_int( const std::string & s, int & result )
   {
-  if( !s.size() ) return 0;
+  if( s.empty() ) return 0;
   const char * const str = s.c_str();
   char * tail;
   errno = 0;
-  const long tmp = std::strtol( str, &tail, 10 );
+  const long tmp = std::strtol( str, &tail, 0 );
   if( errno || tail == str || tmp < -INT_MAX || tmp > INT_MAX ) return 0;
   result = tmp;
   const int c = tail - str;
@@ -371,7 +372,7 @@ int RC::process_rcfile( const std::string & name )
                     name.c_str(), line );
       retval = 2;
       }
-    if( len == 0 || std::isspace( buf[0] ) ) continue;
+    if( len == 0 || ISO_8859::isspace( buf[0] ) ) continue;
     if( buf[0] != '-' )		// Create new file name dependent options
       {
       const char * const regex = std::strtok( buf, delimiters );
@@ -386,7 +387,7 @@ int RC::process_rcfile( const std::string & name )
       const char * const arg = std::strtok( 0, delimiters );
       char msg[80];
       snprintf( msg, sizeof msg, "%s%s %d: ",
-                ( retval ? "" : "\n" ), name.c_str(), line );
+                retval ? "" : "\n", name.c_str(), line );
       const Arg_parser parser( opt, arg, options );
       if( parser.error().size() || !parser.arguments() || !parser.code( 0 ) )
         { std::fprintf( stderr, "%s%s\n", msg, parser.error().c_str() );
