@@ -1,6 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ namespace Regex {
 //
 bool find( const Basic_buffer & buffer, Point & p1, Point & p2,
            const std::string & regex, std::vector< std::string > & pieces,
-           const bool icase = false, const bool backward = false ) throw();
+           const bool icase = false, const bool backward = false );
 
 
 // Deletes block at [p1,p2) and replaces it with the text produced from
@@ -35,12 +35,12 @@ bool find( const Basic_buffer & buffer, Point & p1, Point & p2,
 //
 bool replace( Buffer & buffer, const Point & p1, Point & p2,
               const std::string & regex,
-              const std::vector< std::string > & pieces ) throw();
+              const std::vector< std::string > & pieces );
 
 
 // returns true if 'name' matches 'regex'
 //
 bool match_filename( const std::string & regex, const std::string & name,
-                     unsigned int i1 = 0, unsigned int i2 = 0 ) throw();
+                     unsigned i1 = 0, unsigned i2 = 0 );
 
 } // end namespace Regex
