@@ -1,6 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
-    Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ int kbhit();			// Returns -1 if no key is available.
 
 // Wait for a keystroke. Returns -1 if no key is available.
 // The clock only blinks when waiting here for a keystroke.
-// The screen is also only refreshed here (and in clock_handler).
+// The screen is mainly refreshed here (and in clock_handler).
 // This function is the real "main loop" of the program.
 //
 int wait_kbhit( const int new_clock_limit = -1, const bool get_key = true,
@@ -68,6 +68,7 @@ bool out_line( const std::string & s, const int line, const bool shift = false,
 bool out_menu_line( const std::string & s, const int line,
                     const int hbegin = -1, const int hend = -1 );
 void repaint( const Buffer * const bufp = 0 );
+void show_feedback( const std::string & s );
 int show_message( const std::string & s, const bool get_key = false,
                   const bool take_cursor = false );
 void show_status_lines( const Buffer * const bufp = 0 );
