@@ -1,6 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
-    Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include "buffer.h"
 #include "buffer_handle.h"
 #include "block.h"
+#include "encoding.h"
 #include "iso_8859.h"
 #include "menu.h"
 #include "rc.h"
@@ -100,7 +101,8 @@ void Window::goto_eol()
 
 void Window::goto_begin_of_block()
   {
-  if( Block::bufferp() == &buffer() && buffer().pisvalid( Block::begin() ) )
+  const Buffer & buffer = bhp->buffer();
+  if( Block::bufferp() == &buffer && buffer.pisvalid( Block::begin() ) )
     update_points( Block::begin(), true, true );
   else if( !Block::bufferp() || Block::begin().line < 0 )
     Screen::show_message( "Begin of block is not set" );
@@ -110,7 +112,8 @@ void Window::goto_begin_of_block()
 
 void Window::goto_end_of_block()
   {
-  if( Block::bufferp() == &buffer() && buffer().pisvalid( Block::end() ) )
+  const Buffer & buffer = bhp->buffer();
+  if( Block::bufferp() == &buffer && buffer.pisvalid( Block::end() ) )
     update_points( Block::end(), true, true );
   else if( !Block::bufferp() || Block::end().line < 0 )
     Screen::show_message( "End of block is not set" );
@@ -158,6 +161,7 @@ void Window::goto_column()
 void Window::goto_offset()
   {
   static std::vector< std::string > history;
+  const Buffer & buffer = bhp->buffer();
 
   history.push_back( std::string() );
   if( Screen::get_string( "Go to offset (^C to abort): ", history ) <= 0 )
@@ -168,29 +172,29 @@ void Window::goto_offset()
   if( !RC::parse_int( s, offset ) || ( !relative && offset < 0 ) )
     { Screen::show_message( "Invalid offset value" ); history.pop_back();
       return; }
-  const int cr = buffer().crlf;
+  const int cr = buffer.crlf;
   Point p( 0, 0 );
   if( relative ) p = bhp->pointer;
   p.col += offset;
   if( offset > 0 )
     {
-    while( p.line < buffer().last_line() )
+    while( p.line < buffer.last_line() )
       {
-      const int c = buffer().characters( p.line );
+      const int c = buffer.characters( p.line );
       if( p.col < c + cr ) { p.col = std::min( p.col, c - 1 ); break; }
-      p.col -= ( c + cr ); ++p.line;
+      p.col -= c + cr; ++p.line;
       }
-    if( !bhp->buffer().pisvalid( p ) ) p = bhp->buffer().eof();
+    if( !buffer.pisvalid( p ) ) p = buffer.eof();
     }
   else if( offset < 0 && p.col < 0 )
     {
     while( p.line > 0 )
       {
-      const int c = buffer().characters( --p.line );
+      const int c = buffer.characters( --p.line );
       p.col += c + cr;
       if( p.col >= 0 ) { p.col = std::min( p.col, c - 1 ); break; }
       }
-    if( !bhp->buffer().pisvalid( p ) ) p = bhp->buffer().bof();
+    if( !buffer.pisvalid( p ) ) p = buffer.bof();
     }
   update_points( p, true, true );
   }
@@ -200,7 +204,7 @@ void Window::goto_mark( const int i )
   {
   if( i >= 0 && i <= 9 )
     {
-    const Point & p = buffer().mark[i];
+    const Point & p = bhp->buffer().mark[i];
     if( bhp->buffer().pisvalid( p ) )
       update_points( p, true, true );
     else
@@ -322,11 +326,24 @@ void Window::scroll_vertical( const int lines )
   }
 
 
+void Window::extend_marks()
+  {
+  const int c = bhp->buffer().set_marks();
+  if( c > 0 )
+    {
+    char buf[32];
+    snprintf( buf, sizeof buf, "%d new mark(s) set", c );
+    Screen::show_message( buf );
+    }
+  else Screen::show_message( "No new marks set" );
+  }
+
+
 void Window::set_mark( const int i )
   {
   if( i >= 0 && i <= 9 )
     {
-    buffer().mark[i] = bhp->pointer;
+    bhp->buffer().mark[i] = bhp->pointer;
     char buf[32];
     snprintf( buf, sizeof buf, "Mark %d set", i );
     Screen::show_message( buf );
@@ -345,17 +362,107 @@ void Window::repaint() const
 
 void Window::show_character_info() const
   {
+  const Buffer & buffer = bhp->buffer();
+  const Point p = bhp->pointer;
   std::string s;
-  bhp->buffer().character_info( bhp->pointer, s );
+  int bufsize = 0;
+  char * const buf = Screen::line_buf( &bufsize );
+  const int ch = buffer[p];
+
+  if( ch < 0 && p != buffer.eof() )
+    snprintf( buf, bufsize, "*** Error: Bad Pointer: Line %d  Pos %d",
+              p.line + 1, p.col + 1 );
+  else
+    {
+    int percent = 0;
+    long offset = buffer.character_offset( p, percent );
+    snprintf( buf, bufsize, "Line %d  Pos %d  Offset %ld(0x%lX)  %d%%  ",
+              p.line + 1, p.col + 1, offset, offset, percent );
+    s = buf;
+    if( ch >= 0 )
+      snprintf( buf, bufsize, "Code %d(0x%02X/0%o)", ch, ch ,ch );
+    else
+      snprintf( buf, bufsize, "(End Of File)" );
+    }
+  s += buf;
   Screen::show_message( s );
   }
 
 
 void Window::show_multichar_value( const int size, const bool big_endian ) const
   {
-  std::string s;
-  bhp->buffer().multichar_value( bhp->pointer, s, size, big_endian );
-  Screen::show_message( s );
+  const Buffer & buffer = bhp->buffer();
+  const Point p = bhp->pointer;
+  int bufsize = 0;
+  char * const buf = Screen::line_buf( &bufsize );
+  if( size < 2 || size > 8 )
+    {
+    snprintf( buf, bufsize, "*** Error: Bad Multichar Size: %d", size );
+    Screen::show_message( buf ); return;
+    }
+  std::string text;
+  const int sz = buffer.to_string( p, size, text );
+  int percent = 0;
+  long offset = buffer.character_offset( p, percent );
+
+  if( sz >= size )
+    {
+    unsigned long long value = 0;
+    if( big_endian )
+      for( int i = 0; i < size; ++i )
+        { value <<= 8; value += (unsigned char)text[i]; }
+    else
+      for( int i = size - 1; i >= 0; --i )
+        { value <<= 8; value += (unsigned char)text[i]; }
+    snprintf( buf, bufsize, "Off %ld(0x%lX)  %s_en_%d %llu(0x%0*llX)",
+              offset, offset, big_endian ? "Big" : "Lit",
+              size, value, 2 * size, value );
+    }
+  else if( sz > 0 )
+    snprintf( buf, bufsize, "Offset %ld(0x%lX)  %d%%  (Only %d Characters Left)",
+              offset, offset, percent, sz );
+  else
+    snprintf( buf, bufsize, "Offset %ld(0x%lX)  %d%%  (End Of File)",
+              offset, offset, percent );
+  Screen::show_message( buf );
+  }
+
+
+void Window::show_utf8_code()
+  {
+  const Buffer & buffer = bhp->buffer();
+  Point p = bhp->pointer;
+  const int byte_code = buffer[p];
+  int code = byte_code, len = 1;
+  if( code >= 128 )					// multibyte code
+    {
+    int utf8_offset = 0;
+    while( ( code & 0xC0 ) == 0x80 && p.col > 0 && utf8_offset < 5 )
+      {
+      --p.col; ++utf8_offset; code = buffer[p];		// seek to first byte
+      if( code < 128 ) break;
+      }
+    code = Encoding::utf8_to_ucs( buffer.get_line( p.line ), p.col, &len );
+    if( code < 0 || len <= utf8_offset ) code = -1;
+    }
+
+  int bufsize = 0;
+  char * const buf = Screen::line_buf( &bufsize );
+  if( code >= 0 )
+    {
+    if( p != bhp->pointer ) update_points( p );
+    snprintf( buf, bufsize, "UTF-8 Code %d(0x%04X)  Length %d",
+              code, code, len );
+    }
+  else if( byte_code >= 0 )
+    snprintf( buf, bufsize, "Invalid UTF-8 Byte %d(0x%02X)",
+              byte_code, byte_code );
+  else if( p == buffer.eof() )
+    snprintf( buf, bufsize, "(End Of File)" );
+  else
+    snprintf( buf, bufsize, "*** Error: Bad Pointer: Line %d  Pos %d",
+              p.line + 1, p.col + 1 );
+  Screen::show_message( buf );
   }
 
 
@@ -364,33 +471,34 @@ void Window::show_multichar_value( const int size, const bool big_endian ) const
 void Window::show_status_line( const char * const prefix ) const
   {
   if( top_line_ < 0 ) return;
+  const Buffer & buffer = bhp->buffer();
   int size = 0;
   char * const buf = Screen::line_buf( &size );
   int offset = size - 41;
   const int line = bhp->pcursor.line + 1, col = bhp->pcursor.col + 1;
   for( int i = line; i > 9999; i /= 10 ) --offset;
   for( int i = col; i > 999; i /= 10 ) --offset;
-  const char * name = buffer().nename().c_str();
-  const int overlap = buffer().nename().size() + 11 - offset;
+  const char * name = buffer.nename().c_str();
+  const int overlap = buffer.nename().size() + 11 - offset;
   if( overlap > 0 ) name += overlap;
-  const bool modified = buffer().modified();
+  const bool modified = buffer.modified();
   const int len = snprintf( buf, offset + 1, "%-3s %c%c%c%c%c%c %s%s",
                             prefix ? prefix : "   ",
-                            buffer().options.overwrite ? 'O' : 'I',
-                            buffer().options.word_wrap ? 'W' : ' ',
-                            buffer().options.auto_indent ? 'A' : ' ',
-                            buffer().options.read_only ? 'R' : ' ',
+                            buffer.options.overwrite ? 'O' : 'I',
+                            buffer.options.word_wrap ? 'W' : ' ',
+                            buffer.options.auto_indent ? 'A' : ' ',
+                            buffer.options.read_only ? 'R' : ' ',
                             RC::editor_options().rectangle_mode ? 'X' : ' ',
                             modified ? '*' : ' ', name,
                             modified ? " (Modified)" : "" );
   if( len < offset ) std::memset( buf + len, ' ', offset - len );
   snprintf( buf + offset, size - offset,
-            "  Line %-4d Col %-3d %s  F1 for help",
+            "  Line %-4d Col %-3d %5s   F1 for help",
             line, col, Screen::clock_string() );
   if( RC::editor_options().show_code )
     {
     offset = size - 13;
-    const int ch = bhp->buffer()[bhp->pointer];
+    const int ch = buffer[bhp->pointer];
     if( ch >= 0 )
       snprintf( buf + offset, size - offset, "%3d(0x%02X)", ch, ch );
     else
@@ -427,13 +535,13 @@ void Window::update_points( const Point & to, const bool show,
   const Point rc = relative_cursor();
   if( rc.line > height_ - 3 )
     {
-    const int d = ( center ? ( height_ - 2 ) / 2 :
-                    height_ - ( ( c.line >= buffer.last_line() ) ? 2 : 3 ) );
+    const int d = center ? ( height_ - 2 ) / 2 :
+                  height_ - ( ( c.line >= buffer.last_line() ) ? 2 : 3 );
     tl.line = std::max( 0, c.line - d );
     }
   else if( rc.line < 1 )
     {
-    const int d = ( center ? ( height_ - 2 ) / 2 : 1 );
+    const int d = center ? ( height_ - 2 ) / 2 : 1;
     tl.line = std::max( 0, c.line - d );
     }
   const int ahead = std::max( 1, std::min( 8,
