@@ -1,6 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
-    Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+    2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -104,14 +104,14 @@ int Window_vector::load( int new_size )
   {
   const int handles = Bufhandle_vector::handles();
   const bool toggle = ( new_size < 0 && handles > 1 );
-  const bool center = ( data.size() == 0 );		// center all on init
+  const bool center = data.empty();			// center all on init
 
   if( !handles ) { curwin_ = 0; data.clear(); return 0; }
   if( new_size == 0 ) new_size = data.size();
   else if( new_size < 0 )
     {
     for( int i = 0; i < curwin_; ++i ) Bufhandle_vector::next();
-    new_size = ( ( data.size() != 1 ) ? 1 : handles );
+    new_size = ( data.size() != 1 ) ? 1 : handles;
     curwin_ = 0;
     }
   new_size = std::min( new_size, handles );
@@ -151,13 +151,14 @@ void Window_vector::change_buffer_name()
   if( Menu::get_filename( "New buffer name (^C to abort): ", name ) < 0 ||
       name == buffer.name() )
     return;
-  for( int i = 0; i < Bufhandle_vector::handles(); ++i )
-    if( name == Bufhandle_vector::handle( i ).buffer().name() &&
-        &buffer != &Bufhandle_vector::handle( i ).buffer() )
-      {
-      std::string s( "File '" ); s += name; s += "' is already being edited";
-      Screen::show_message( s ); return;
-      }
+  if( name.size() )			// allow multiple unnamed buffers
+    for( int i = 0; i < Bufhandle_vector::handles(); ++i )
+      if( name == Bufhandle_vector::handle( i ).buffer().name() &&
+          &buffer != &Bufhandle_vector::handle( i ).buffer() )
+        {
+        std::string s( "File '" ); s += name; s += "' is already being edited";
+        Screen::show_message( s ); return;
+        }
   buffer.name( name );
   Screen::show_status_lines( &buffer );
   }
@@ -229,9 +230,9 @@ int Window_vector::close( bool abort, const bool quiet )
       {
       if( read_only( bh.buffer() ) ) return -1;
       std::string name( bh.buffer().name() );
-      if( exit_ask || !name.size() )
+      if( exit_ask || name.empty() )
         while( !Menu::get_filename( "Name of file to save (^C to abort): ", name ) ) ;
-      if( !name.size() ) return -1;
+      if( name.empty() ) return -1;
       try { if( !bh.buffer().save( &name ) ) return -1; }
       catch( Buffer::Error e )
         { if( !quiet ) Screen::show_message( e.msg ); return -1; }
@@ -601,7 +602,7 @@ void Window_vector::reformat()
     for( Point p = bot; p < eop; )
       {
       unsigned char ch = buffer.pgetc( p );
-      if( !std::isspace( ch ) )
+      if( !ISO_8859::isspace( ch ) )
         { tmp.pputc( dummy, ch ); dummy = tmp.eof(); space = false; }
       else if( !space ) { tmp.pputc( dummy, ' ' ); space = true; }
       if( Screen::kbhit() >= 0 && Screen::wait_kbhit() == 3 )
@@ -689,10 +690,37 @@ void Window_vector::decode_base64()
   std::string in, out;
   Block::bufferp()->to_string( Block::begin(), Block::end(), in );
   for( int i = in.size() - 1; i >= 0; --i )
-    if( std::isspace( in[i] ) )
+    if( ISO_8859::isspace( in[i] ) )
       in.erase( i, 1 );
-  if( !Encoding::base64_decode( in, out ) )
-    { Screen::show_message( "Invalid Base64 data in block" ); return; }
+  int idx;
+  if( !Encoding::base64_decode( in, out, &idx ) )
+    {
+    Block::disable_marking();
+    Point p = Block::begin();
+    while( idx > 0 ) if( !ISO_8859::isspace( buffer.pgetc( p ) ) ) --idx;
+    while( ISO_8859::isspace( buffer[p] ) ) buffer.pnext( p );
+    if( idx == 0 && buffer.pisvalid( p ) )
+      data[curwin_].update_points( p, true, true );
+    Screen::show_message( "Invalid Base64 data in block" );
+    return;
+    }
+  Basic_buffer tmp;
+  Point p = tmp.bof();
+  for( unsigned i = 0; i < out.size(); ++i )
+    tmp.pputc( p, out[i] );
+  replace_and_repaint( buffer, tmp );
+  }
+
+
+void Window_vector::encode_utf8()
+  {
+  Buffer & buffer = data[curwin_].buffer();
+  if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
+      read_only( buffer ) ) return;
+  std::string in, out;
+  Block::bufferp()->to_string( Block::begin(), Block::end(), in );
+  if( !Encoding::utf8_encode( in, out ) )
+    { Screen::show_message( "Text in block is already UTF-8 encoded" ); return; }
   Basic_buffer tmp;
   Point p = tmp.bof();
   for( unsigned i = 0; i < out.size(); ++i )
@@ -709,12 +737,40 @@ void Window_vector::decode_quoted_printable_utf8( const bool mode_q )
   std::string in, out;
   Block::bufferp()->to_string( Block::begin(), Block::end(), in );
   if( mode_q )
-    { if( !Encoding::quoted_printable_decode( in, out ) )
-      { Screen::show_message( "Invalid Quoted-Printable data in block" ); return; } }
+    {
+    int idx;
+    if( !Encoding::quoted_printable_decode( in, out, &idx ) )
+      {
+      Block::disable_marking();
+      Point p = Block::begin();
+      if( buffer.pisvalid( p ) && buffer.pseek( p, idx ) )
+        data[curwin_].update_points( p, true, true );
+      Screen::show_message( "Invalid Quoted-Printable data in block" );
+      return;
+      }
+    }
   else
-    { if( !Encoding::utf8_decode( in, out ) )
-      { Screen::show_message( "Invalid UTF-8 data in block" ); return; } }
-  if( in.size() == out.size() )
+    {
+    int idx;
+    const int tmp = Encoding::utf8_decode( in, out, &idx );
+    if( tmp != 0 )
+      {
+      Block::disable_marking();
+      Point p = Block::begin();
+      if( buffer.pisvalid( p ) && buffer.pseek( p, idx ) )
+        data[curwin_].update_points( p, true, true );
+      if( tmp > 0 )
+        {
+        char buf[64];
+        snprintf( buf, sizeof buf, "UTF-8 code 'U%04X' out of range in block", tmp );
+        Screen::show_message( buf );
+        }
+      else
+        Screen::show_message( "Invalid UTF-8 data in block" );
+      return;
+      }
+    }
+  if( in == out )
     { Screen::show_message( "Text in block is already decoded" ); return; }
   Basic_buffer tmp;
   Point p = tmp.bof();
@@ -809,7 +865,7 @@ const char * Window_vector::copyright_update()
   for( unsigned i = 0; i < year_string.size(); ++i )
     if( !std::isdigit( year_string[i] ) )
       year_string.clear();
-  if( !year_string.size() || year_string[0] == '0' ) return "Invalid year";
+  if( year_string.empty() || year_string[0] == '0' ) return "Invalid year";
   year_string.insert( 0, "\\<" ); year_string += "\\>";
   bool ask = true, found = false, modified = false;
   for( int i = 0; i < Bufhandle_vector::handles(); next(), ++i )
@@ -917,7 +973,7 @@ const char * Window_vector::search( const int abort_key, const bool again,
   static bool icase = false;
   static bool replace = false;
 
-  if( !again || !search_string.size() )
+  if( !again || search_string.empty() )
     {
     search_history.push_back( s_ini );
     int size = Screen::get_string( "Find (^C to abort): ", search_history,
@@ -959,12 +1015,11 @@ const char * Window_vector::search( const int abort_key, const bool again,
       const int len = Screen::get_string( "Replace with (^C to abort): ",
                                           replace_history, abort_key );
       if( len < 0 ) { replace = false; return 0; }
-      if( len > 0 )
-        { replace_string = replace_history.back();
-          ISO_8859::deescapize( replace_string ); }
+      if( len > 0 ) replace_string = replace_history.back();
       }
     }
   if( toggle ) backward = !backward;
+  Screen::show_feedback( "searching..." );
 
   Point p = curwin().pointer();			// search point
   if( block )
@@ -975,7 +1030,7 @@ const char * Window_vector::search( const int abort_key, const bool again,
     }
   bool found = false, ask = true, found_any = false, wrapped = false;
   bool empty_replace = false;
-  const int handles = ( global ? Bufhandle_vector::handles() : 1 );
+  const int handles = global ? Bufhandle_vector::handles() : 1;
   for( int i = handles - 1; i >= 0 && !found; --i )
     {
     Window & w = data[curwin_];
